@@ -9,7 +9,7 @@
  * published by the Free Software Foundation; either version 2 of the
  * License, or (at your option) any later version.
  *
- * The Rhythmbox authors hereby grants permission for non-GPL compatible
+ * The Rhythmbox authors hereby grant permission for non-GPL compatible
  * GStreamer plugins to be used and distributed together with GStreamer
  * and Rhythmbox. This permission is above and beyond the permissions granted
  * by the GPL license by which Rhythmbox is covered. If you modify this code
@@ -34,36 +34,21 @@
 #include <string.h>
 #include <math.h>
 #include <time.h>
-#include <sys/vfs.h>
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <fcntl.h>
 
 #include <glib/gi18n.h>
 #include <gdk/gdk.h>
-#include <libgnomevfs/gnome-vfs-utils.h>
 #include <gst/gst.h>
 
-#include <nautilus-burn-drive.h>
-#include <nautilus-burn-recorder.h>
-#ifndef NAUTILUS_BURN_CHECK_VERSION 	 
-#define NAUTILUS_BURN_CHECK_VERSION(a,b,c) FALSE 	 
-#endif
-
-#if NAUTILUS_BURN_CHECK_VERSION(2,15,3)
 #include <nautilus-burn.h>
-#endif
 
 #include "rb-recorder.h"
 
 #include "rb-debug.h"
 #include "rb-marshal.h"
 
-#ifndef HAVE_BURN_DRIVE_UNREF
-#define nautilus_burn_drive_unref nautilus_burn_drive_free
-#define nautilus_burn_drive_ref nautilus_burn_drive_copy
-#endif
-
 static void rb_recorder_class_init (RBRecorderClass *klass);
 static void rb_recorder_init       (RBRecorder      *recorder);
 static void rb_recorder_finalize   (GObject         *object);
@@ -72,7 +57,7 @@ struct _RBRecorderPrivate {
         char       *src_uri;
         char       *dest_file;
         char       *tmp_dir;
-        
+
         GstElement *pipeline;
         GstElement *decoder;
         GstElement *src;
@@ -82,9 +67,9 @@ struct _RBRecorderPrivate {
         GstElement *encoder;
         GstElement *sink;
 
-	GstElement *capsfilter;
-	GstElement *audioconvert2;
-	gboolean    got_audio_pad;
+        GstElement *capsfilter;
+        GstElement *audioconvert2;
+        gboolean    got_audio_pad;
 
         guint       tick_timeout_id;
 
@@ -190,14 +175,14 @@ rb_recorder_class_init (RBRecorderClass *klass)
                               G_TYPE_BOOLEAN,
                               G_TYPE_BOOLEAN,
                               G_TYPE_BOOLEAN);
-	rb_recorder_signals [WARN_DATA_LOSS] =
-		g_signal_new ("warn-data-loss",
-			      G_TYPE_FROM_CLASS (klass),
-			      G_SIGNAL_RUN_LAST,
-			      0,
-			      NULL, NULL,
-			      rb_marshal_INT__VOID,
-			      G_TYPE_INT, 0);
+        rb_recorder_signals [WARN_DATA_LOSS] =
+                g_signal_new ("warn-data-loss",
+                              G_TYPE_FROM_CLASS (klass),
+                              G_SIGNAL_RUN_LAST,
+                              0,
+                              NULL, NULL,
+                              rb_marshal_INT__VOID,
+                              G_TYPE_INT, 0);
         rb_recorder_signals [ERROR] =
                 g_signal_new ("error",
                               G_TYPE_FROM_CLASS (klass),
@@ -218,13 +203,9 @@ rb_recorder_get_default_drive (void)
         NautilusBurnDrive *drive  = NULL;
         GList             *drives = NULL;
 
-#if NAUTILUS_BURN_CHECK_VERSION(2,15,3)
         NautilusBurnDriveMonitor *monitor;
         monitor = nautilus_burn_get_drive_monitor ();
         drives = nautilus_burn_drive_monitor_get_recorder_drives (monitor);
-#else
-        drives = nautilus_burn_drive_get_list (TRUE, FALSE);
-#endif
 
         if (drives) {
                 drive = nautilus_burn_drive_ref ((NautilusBurnDrive*) drives->data);
@@ -280,7 +261,7 @@ rb_recorder_gst_free_pipeline (RBRecorder *recorder)
                         recorder->priv->start_timer = NULL;
                 }
         }
-	recorder->priv->got_audio_pad = FALSE;
+        recorder->priv->got_audio_pad = FALSE;
 
         gst_element_set_state (recorder->priv->pipeline, GST_STATE_NULL);
 
@@ -293,16 +274,16 @@ add_track (RBRecorder *recorder,
            const char *cdtext)
 {
         NautilusBurnRecorderTrack *track;
-        char                      *filename;
+        char                      *file;
 
         g_return_val_if_fail (RB_IS_RECORDER (recorder), FALSE);
 
-        filename = g_strdup (recorder->priv->dest_file);
+        file = g_strdup (recorder->priv->dest_file);
 
         track = g_new0 (NautilusBurnRecorderTrack, 1);
 
         track->type = NAUTILUS_BURN_RECORDER_TRACK_TYPE_AUDIO;
-        track->contents.audio.filename = filename;
+        track->contents.audio.filename = file;
         if (cdtext) {
                 track->contents.audio.cdtext = g_strdup (cdtext);
         }
@@ -314,103 +295,104 @@ add_track (RBRecorder *recorder,
 
 static void
 rb_recorder_gst_signal_error (RBRecorder *recorder,
-			      const char *msg)
+                              const char *msg)
 {
-	GError *error;
+        GError *error;
 
-	g_object_ref (recorder);
-	error = g_error_new_literal (RB_RECORDER_ERROR,
-				     RB_RECORDER_ERROR_GENERAL,
-				     msg);
-	g_signal_emit (G_OBJECT (recorder),
-		       rb_recorder_signals [ERROR],
-		       0,
-		       error);
+        g_object_ref (recorder);
+        error = g_error_new_literal (RB_RECORDER_ERROR,
+                                     RB_RECORDER_ERROR_GENERAL,
+                                     msg);
+        g_signal_emit (G_OBJECT (recorder),
+                       rb_recorder_signals [ERROR],
+                       0,
+                       error);
 
-	/* close if not already closing */
-	if (recorder->priv->src_uri != NULL) {
-		rb_recorder_close (recorder, NULL);
+        /* close if not already closing */
+        if (recorder->priv->src_uri != NULL) {
+                rb_recorder_close (recorder, NULL);
         }
 
-	g_object_unref (recorder);
-	g_error_free (error);
+        g_object_unref (recorder);
+        g_error_free (error);
 }
 
 static gboolean
 pipe_message (GstBus *bus, GstMessage *message, RBRecorder *recorder)
 {
-	switch (message->type) {
-	case GST_MESSAGE_EOS: 
-		rb_debug ("EOS");
+        switch (message->type) {
+        case GST_MESSAGE_EOS:
+                rb_debug ("EOS");
 
-		if (recorder->priv->pipeline)
-			gst_element_set_state (recorder->priv->pipeline, GST_STATE_NULL);
+                if (recorder->priv->pipeline)
+                        gst_element_set_state (recorder->priv->pipeline, GST_STATE_NULL);
 
-		g_signal_emit (G_OBJECT (recorder), 
-			       rb_recorder_signals [EOS], 0);
-		break;
-	case GST_MESSAGE_ERROR:
-	{
-		GError *error;
-		gchar *debug;
+                g_signal_emit (G_OBJECT (recorder),
+                               rb_recorder_signals [EOS], 0);
+                break;
+        case GST_MESSAGE_ERROR:
+        {
+                GError *error;
+                gchar *debug;
 
-		rb_debug ("Error");
-		gst_message_parse_error (message, &error, &debug);
-		if (error) {
-			rb_recorder_gst_signal_error (recorder, error->message);
-			g_error_free (error);
-		} else {
-			rb_recorder_gst_signal_error (recorder, NULL);
-		}
+                rb_debug ("Error");
+                gst_message_parse_error (message, &error, &debug);
+                if (error) {
+                        rb_recorder_gst_signal_error (recorder, error->message);
+                        g_error_free (error);
+                } else {
+                        rb_recorder_gst_signal_error (recorder, NULL);
+                }
 
-		g_free (debug);
-		break;
-	}
-	default:
-		break;
-	}
+                g_free (debug);
+                break;
+        }
+        default:
+                break;
+        }
 
-	return TRUE;
+        return TRUE;
 }
- 
+
 static void
 rb_recorder_new_pad_cb (GstElement *decodebin,
-			GstPad     *pad,
-			gboolean    last,
-			RBRecorder *recorder)
+                        GstPad     *pad,
+                        gboolean    last,
+                        RBRecorder *recorder)
 {
-	GstCaps      *caps;
-	GstStructure *str;
-	GstPad       *audio_pad;
+        GstCaps      *caps;
+        GstStructure *str;
+        GstPad       *audio_pad;
 
-	audio_pad = gst_element_get_pad (recorder->priv->audioconvert, "sink");
+        audio_pad = gst_element_get_static_pad (recorder->priv->audioconvert, "sink");
 
-	/* Only link once. */
-	if (GST_PAD_IS_LINKED (audio_pad))
-		return;
+        /* Only link once. */
+        if (GST_PAD_IS_LINKED (audio_pad))
+                return;
 
-	/* Only link audio. */
-	caps = gst_pad_get_caps (pad);
-	str = gst_caps_get_structure (caps, 0);
-	if (! g_strrstr (gst_structure_get_name (str), "audio")) {
-		gst_caps_unref (caps);
-		return;
-	}
+        /* Only link audio. */
+        caps = gst_pad_get_caps (pad);
+        str = gst_caps_get_structure (caps, 0);
+        if (! g_strrstr (gst_structure_get_name (str), "audio")) {
+                gst_caps_unref (caps);
+                return;
+        }
 
-	gst_caps_unref (caps);
-	if (gst_pad_link (pad, audio_pad)) {
-		recorder->priv->got_audio_pad = TRUE;
+        gst_caps_unref (caps);
+        if (gst_pad_link (pad, audio_pad)) {
+                recorder->priv->got_audio_pad = TRUE;
         }
 }
 
 static void
 rb_recorder_construct (RBRecorder *recorder,
-                       const char *uri,
+                       const char *src_uri,
+                       const char *dest_file,
                        GError    **error)
 {
         char    *element_name = NULL;
         GstCaps *filtercaps;
-	GstBus  *bus;
+        GstBus  *bus;
 
 #define MAKE_ELEMENT_OR_LOSE(NAME, NICE) G_STMT_START {                 \
                 element_name = #NAME ;                                  \
@@ -439,15 +421,22 @@ rb_recorder_construct (RBRecorder *recorder,
                 return;
         }
 
-	bus = gst_element_get_bus (recorder->priv->pipeline);
-	gst_bus_add_watch (bus, (GstBusFunc) pipe_message, recorder);
-	gst_object_unref (bus);
+        bus = gst_element_get_bus (recorder->priv->pipeline);
+        gst_bus_add_watch (bus, (GstBusFunc) pipe_message, recorder);
+        gst_object_unref (bus);
 
         /* Construct elements */
 
         /* The source */
+        recorder->priv->src = gst_element_make_from_uri (GST_URI_SRC, src_uri, NULL);
+        if (!recorder->priv->src) {
+                goto missing_element;
+        }
 
-        MAKE_ELEMENT_OR_LOSE(gnomevfssrc, src);
+        if (g_object_class_find_property (G_OBJECT_GET_CLASS (recorder->priv->src),
+                                          "iradio-mode")) {
+                g_object_set (recorder->priv->src, "iradio-mode", FALSE, NULL);
+        }
         gst_bin_add (GST_BIN (recorder->priv->pipeline), recorder->priv->src);
 
         /* The queue */
@@ -457,10 +446,10 @@ rb_recorder_construct (RBRecorder *recorder,
 
         MAKE_ELEMENT_OR_LOSE(decodebin, decoder);
         gst_bin_add (GST_BIN (recorder->priv->pipeline), recorder->priv->decoder);
-	
-	g_signal_connect_object (G_OBJECT (recorder->priv->decoder), 
-				 "new_decoded_pad", G_CALLBACK (rb_recorder_new_pad_cb), 
-				 recorder, 0);
+
+        g_signal_connect_object (G_OBJECT (recorder->priv->decoder),
+                                 "new_decoded_pad", G_CALLBACK (rb_recorder_new_pad_cb),
+                                 recorder, 0);
 
         MAKE_ELEMENT_OR_LOSE(audioconvert, audioconvert);
         gst_bin_add (GST_BIN (recorder->priv->pipeline), recorder->priv->audioconvert);
@@ -471,15 +460,18 @@ rb_recorder_construct (RBRecorder *recorder,
         MAKE_ELEMENT_OR_LOSE(audioconvert, audioconvert2);
         gst_bin_add (GST_BIN (recorder->priv->pipeline), recorder->priv->audioconvert2);
 
-	MAKE_ELEMENT_OR_LOSE(capsfilter, capsfilter);
-	gst_bin_add (GST_BIN (recorder->priv->pipeline), recorder->priv->capsfilter);
-	
+        MAKE_ELEMENT_OR_LOSE(capsfilter, capsfilter);
+        gst_bin_add (GST_BIN (recorder->priv->pipeline), recorder->priv->capsfilter);
+
         MAKE_ELEMENT_OR_LOSE(wavenc, encoder);
         gst_bin_add (GST_BIN (recorder->priv->pipeline), recorder->priv->encoder);
 
         /* Output sink */
-
-        MAKE_ELEMENT_OR_LOSE(gnomevfssink, sink);
+        recorder->priv->sink = gst_element_factory_make ("filesink", "sink");
+        if (recorder->priv->sink == NULL) {
+                goto missing_element;
+        }
+        g_object_set (recorder->priv->sink, "location", dest_file, NULL);
         gst_bin_add (GST_BIN (recorder->priv->pipeline), recorder->priv->sink);
 
         filtercaps = gst_caps_new_simple ("audio/x-raw-int",
@@ -487,22 +479,22 @@ rb_recorder_construct (RBRecorder *recorder,
                                           "rate",     G_TYPE_INT, 44100,
                                           "width",    G_TYPE_INT, 16,
                                           "depth",    G_TYPE_INT, 16,
-					  "endianness",	G_TYPE_INT, G_LITTLE_ENDIAN,
+                                          "endianness", G_TYPE_INT, G_LITTLE_ENDIAN,
                                           NULL);
-	g_object_set (recorder->priv->capsfilter, "caps",  filtercaps, NULL);
+        g_object_set (recorder->priv->capsfilter, "caps",  filtercaps, NULL);
 
-	gst_element_link_many (recorder->priv->src,
-			       recorder->priv->typefind,
-			       recorder->priv->decoder,
-			       NULL);
+        gst_element_link_many (recorder->priv->src,
+                               recorder->priv->typefind,
+                               recorder->priv->decoder,
+                               NULL);
 
-	gst_element_link_many (recorder->priv->audioconvert,
-			       recorder->priv->audioscale,
-			       recorder->priv->audioconvert2,
-			       recorder->priv->capsfilter,
-			       recorder->priv->encoder,
-			       recorder->priv->sink,
-			       NULL);
+        gst_element_link_many (recorder->priv->audioconvert,
+                               recorder->priv->audioscale,
+                               recorder->priv->audioconvert2,
+                               recorder->priv->capsfilter,
+                               recorder->priv->encoder,
+                               recorder->priv->sink,
+                               NULL);
 
         rb_debug ("Pipeline construction complete");
         return;
@@ -583,19 +575,19 @@ tick_timeout_cb (RBRecorder *recorder)
         double elapsed;
         double secs;
         GstFormat format = GST_FORMAT_BYTES;
-	GstState  state;
+        GstState  state;
 
         g_return_val_if_fail (recorder != NULL, FALSE);
         g_return_val_if_fail (RB_IS_RECORDER (recorder), FALSE);
         g_return_val_if_fail (recorder->priv != NULL, FALSE);
         g_return_val_if_fail (recorder->priv->pipeline != NULL, FALSE);
 
-	if (!gst_element_get_state (recorder->priv->pipeline, &state,  NULL, 3 * GST_SECOND)) {
-		g_warning (_("Could not retrieve state from processing pipeline"));
-		return TRUE;
-	}
+        if (!gst_element_get_state (recorder->priv->pipeline, &state,  NULL, 3 * GST_SECOND)) {
+                g_warning (_("Could not retrieve state from processing pipeline"));
+                return TRUE;
+        }
 
-	if (state != GST_STATE_PLAYING) {		
+        if (state != GST_STATE_PLAYING) {
                 recorder->priv->tick_timeout_id = 0;
                 if (recorder->priv->start_timer) {
                         g_timer_destroy (recorder->priv->start_timer);
@@ -604,7 +596,8 @@ tick_timeout_cb (RBRecorder *recorder)
                 return FALSE;
         }
 
-        if (!gst_element_query_position (recorder->priv->src, &format, &position) || !gst_element_query_duration (recorder->priv->src, &format, &total)) {
+        if (!gst_element_query_position (recorder->priv->src, &format, &position) ||
+            !gst_element_query_duration (recorder->priv->src, &format, &total)) {
                 g_warning (_("Could not get current track position"));
                 return TRUE;
         }
@@ -645,7 +638,7 @@ rb_recorder_sync_pipeline (RBRecorder *recorder,
         g_return_val_if_fail (RB_IS_RECORDER (recorder), FALSE);
         g_return_val_if_fail (recorder->priv != NULL, FALSE);
         g_return_val_if_fail (recorder->priv->pipeline != NULL, FALSE);
-        
+
         rb_debug ("Syncing pipeline");
         if (recorder->priv->playing) {
                 rb_debug ("Playing pipeline");
@@ -657,7 +650,7 @@ rb_recorder_sync_pipeline (RBRecorder *recorder,
                         return FALSE;
                 }
 
-		recorder->priv->tick_timeout_id = g_timeout_add (200, (GSourceFunc)tick_timeout_cb, recorder);
+                recorder->priv->tick_timeout_id = g_timeout_add (200, (GSourceFunc)tick_timeout_cb, recorder);
         } else {
                 rb_debug ("Pausing pipeline");
                 if (gst_element_set_state (recorder->priv->pipeline, GST_STATE_PAUSED) == GST_STATE_CHANGE_FAILURE) {
@@ -688,7 +681,7 @@ rb_recorder_close (RBRecorder *recorder,
 
         rb_debug ("Closing rb_recorder");
 
-	recorder->priv->got_audio_pad = FALSE;
+        recorder->priv->got_audio_pad = FALSE;
         recorder->priv->playing = FALSE;
         g_free (recorder->priv->src_uri);
         recorder->priv->src_uri = NULL;
@@ -723,7 +716,7 @@ get_dest_from_uri (const char *tmp_dir,
                    const char *src_uri)
 {
         char *lock_filename;
-        char *filename;
+        char *wav_filename;
         int   fd;
 
         lock_filename = g_build_filename (tmp_dir, "rb-burn-tmp.XXXXXX", NULL);
@@ -733,10 +726,10 @@ get_dest_from_uri (const char *tmp_dir,
         /* keep empty file around until finalize
            it will serve as a lock file to protect our new filename */
 
-        filename = g_strdup_printf ("%s.wav", lock_filename);
+        wav_filename = g_strconcat (lock_filename, ".wav", NULL);
         g_free (lock_filename);
 
-        return filename;
+        return wav_filename;
 }
 
 void
@@ -747,7 +740,7 @@ rb_recorder_open (RBRecorder   *recorder,
 {
         char    *dest_file;
         gboolean audiocd_mode = src_uri && g_str_has_prefix (src_uri, "audiocd://");
-        
+
         g_return_if_fail (recorder != NULL);
         g_return_if_fail (RB_IS_RECORDER (recorder));
         g_return_if_fail (recorder->priv != NULL);
@@ -760,27 +753,21 @@ rb_recorder_open (RBRecorder   *recorder,
                 return;
         }
 
-        rb_recorder_construct (recorder, src_uri, error);
+        dest_file = get_dest_from_uri (recorder->priv->tmp_dir, src_uri);
+
+        rb_recorder_construct (recorder, src_uri, dest_file, error);
         if (error && *error) {
+                g_free (dest_file);
                 return;
         }
-                
-	recorder->priv->got_audio_pad = FALSE;
 
-        g_object_set (G_OBJECT (recorder->priv->src), "iradio-mode", FALSE, NULL);
-        gst_element_set_state (recorder->priv->src, GST_STATE_NULL);
-        g_object_set (G_OBJECT (recorder->priv->src), "location", src_uri, NULL);
+        recorder->priv->got_audio_pad = FALSE;
 
         g_free (recorder->priv->src_uri);
         recorder->priv->src_uri = g_strdup (src_uri);
 
-        dest_file = get_dest_from_uri (recorder->priv->tmp_dir, src_uri);
-        gst_element_set_state (recorder->priv->sink, GST_STATE_NULL);
-        g_object_set (G_OBJECT (recorder->priv->sink), "location", dest_file, NULL);
-        
         g_free (recorder->priv->dest_file);
-        recorder->priv->dest_file = g_strdup (dest_file);
-        g_free (dest_file);
+        recorder->priv->dest_file = dest_file;
 
         recorder->priv->playing = FALSE;
 
@@ -832,16 +819,6 @@ rb_recorder_pause (RBRecorder *recorder,
         rb_recorder_sync_pipeline (recorder, NULL);
 }
 
-#if !NAUTILUS_BURN_CHECK_VERSION(2,15,3)
-static const char *
-nautilus_burn_drive_get_device (NautilusBurnDrive *drive)
-{
-	g_return_val_if_fail (drive != NULL, NULL);
-
-	return drive->device;
-}
-#endif
-
 char *
 rb_recorder_get_device (RBRecorder  *recorder,
                         GError     **error)
@@ -873,12 +850,7 @@ rb_recorder_set_device (RBRecorder  *recorder,
                         const char  *device,
                         GError     **error)
 {
-#if NAUTILUS_BURN_CHECK_VERSION(2,15,3)
         NautilusBurnDriveMonitor *monitor;
-#else
-        GList *drives;
-        GList *tmp;
-#endif
         int type;
 
         g_return_val_if_fail (recorder != NULL, FALSE);
@@ -896,29 +868,11 @@ rb_recorder_set_device (RBRecorder  *recorder,
 
         type = 0;
 
-#if NAUTILUS_BURN_CHECK_VERSION(2,15,3)
         monitor = nautilus_burn_get_drive_monitor ();
         recorder->priv->drive = nautilus_burn_drive_monitor_get_drive_for_device (monitor, device);
         if (recorder->priv->drive != NULL) {
                 type = nautilus_burn_drive_get_drive_type (recorder->priv->drive);
         }
-#else
-        drives = nautilus_burn_drive_get_list (TRUE, FALSE);
-
-        for (tmp = drives; tmp != NULL; tmp = tmp->next) {
-                NautilusBurnDrive *drive = (NautilusBurnDrive *) tmp->data;
-
-                if (strcmp (drive->device, device) == 0) {
-                        recorder->priv->drive = drive;
-                        break;
-                }
-
-                nautilus_burn_drive_unref (drive);
-        }
-        g_list_free (drives);
-
-        type = recorder->priv->drive->type;
-#endif
 
         if (! recorder->priv->drive) {
                 g_set_error (error,
@@ -973,8 +927,6 @@ rb_recorder_action_changed_cb (NautilusBurnRecorder        *cdrecorder,
                        action);
 }
 
-#ifdef NAUTILUS_BURN_DRIVE_SIZE_TO_TIME
-/* If nautilus-cd-burner >= 2.12 */
 static void
 rb_recorder_burn_progress_cb (NautilusBurnRecorder *cdrecorder,
                               gdouble               fraction,
@@ -989,23 +941,6 @@ rb_recorder_burn_progress_cb (NautilusBurnRecorder *cdrecorder,
                        fraction,
                        secs);
 }
-#else
-/* If nautilus-cd-burner == 2.10 */
-static void
-rb_recorder_burn_progress_cb (NautilusBurnRecorder *cdrecorder,
-                              gdouble               fraction,
-                              gpointer              data)
-{
-        RBRecorder *recorder = (RBRecorder*) data;
-        long        secs     = -1;
-
-        g_signal_emit (recorder,
-                       rb_recorder_signals [BURN_PROGRESS_CHANGED],
-                       0,
-                       fraction,
-                       secs);
-}
-#endif
 
 static gboolean
 rb_recorder_insert_cd_request_cb (NautilusBurnRecorder *cdrecorder,
@@ -1033,39 +968,20 @@ rb_recorder_warn_data_loss_cb (NautilusBurnRecorder *cdrecorder,
                                RBRecorder           *recorder)
 {
         int res = 0;
-        int ret = 0;
-        int retry_val;
-        int cancel_val;
-        int erase_val;
 
         g_signal_emit (G_OBJECT (recorder),
                        rb_recorder_signals [WARN_DATA_LOSS],
                        0, &res);
 
-#ifdef NAUTILUS_BURN_DRIVE_SIZE_TO_TIME
-        retry_val  = NAUTILUS_BURN_RECORDER_RESPONSE_RETRY;
-        cancel_val = NAUTILUS_BURN_RECORDER_RESPONSE_CANCEL;
-        erase_val  = NAUTILUS_BURN_RECORDER_RESPONSE_ERASE;
-#else
-        retry_val  = TRUE;
-        cancel_val = TRUE;
-        erase_val  = FALSE;
-#endif
-
         switch (res) {
         case RB_RECORDER_RESPONSE_RETRY:
-                ret = retry_val;
-                break;
+                return NAUTILUS_BURN_RECORDER_RESPONSE_RETRY;
         case RB_RECORDER_RESPONSE_ERASE:
-                ret = erase_val;
-                break;
+                return NAUTILUS_BURN_RECORDER_RESPONSE_ERASE;
         case RB_RECORDER_RESPONSE_CANCEL:
         default:
-                ret = cancel_val;
-                break;
+                return NAUTILUS_BURN_RECORDER_RESPONSE_CANCEL;
         }
-
-        return ret;
 }
 
 char *
@@ -1099,23 +1015,15 @@ rb_recorder_get_media_length (RBRecorder *recorder,
                 g_set_error (error,
                              RB_RECORDER_ERROR,
                              RB_RECORDER_ERROR_INTERNAL,
-                             _("No writable drives found."));
+                             _("No writable drives found"));
 
                 return -1;
         }
 
-#if NAUTILUS_BURN_CHECK_VERSION(2,15,3)
         size = nautilus_burn_drive_get_media_capacity (recorder->priv->drive);
-#else
-        size = nautilus_burn_drive_get_media_size (recorder->priv->drive);
-#endif
 
         if (size > 0) {
-#ifdef NAUTILUS_BURN_DRIVE_SIZE_TO_TIME
                 secs = NAUTILUS_BURN_DRIVE_SIZE_TO_TIME (size);
-#else
-                secs = SIZE_TO_TIME (size);
-#endif
         } else {
                 secs = size;
         }
@@ -1133,23 +1041,23 @@ rb_recorder_get_media_length (RBRecorder *recorder,
 /* Copyright (C) Bastien Nocera */
 /* From xine-lib, whoop */
 typedef struct GNUC_PACKED {
-	gint16   wFormatTag;
-	gint16   nChannels;
-	gint32   nSamplesPerSec;
-	gint32   nAvgBytesPerSec;
-	gint16   nBlockAlign;
-	gint16   wBitsPerSample;
-	gint16   cbSize;
+        gint16   wFormatTag;
+        gint16   nChannels;
+        gint32   nSamplesPerSec;
+        gint32   nAvgBytesPerSec;
+        gint16   nBlockAlign;
+        gint16   wBitsPerSample;
+        gint16   cbSize;
 } waveformat;
 
 #ifndef GNUC_PACKED
 #pragma pack()
 #endif
 
-#define ACB_ERROR_OPEN				-1
-#define ACB_ERROR_NOT_WAVE_TOO_SMALL		-2
-#define ACB_ERROR_NOT_WAVE_FILE			-3
-#define ACB_ERROR_NOT_WAVE_FORMAT		-4
+#define ACB_ERROR_OPEN                          -1
+#define ACB_ERROR_NOT_WAVE_TOO_SMALL            -2
+#define ACB_ERROR_NOT_WAVE_FILE                 -3
+#define ACB_ERROR_NOT_WAVE_FORMAT               -4
 
 /* Copyright (C) Bastien Nocera */
 /* Data from
@@ -1190,7 +1098,7 @@ acb_wave_time (const char *filename)
                 return ACB_ERROR_NOT_WAVE_TOO_SMALL;
         }
 
-	len = GINT_FROM_LE (len);
+        len = GINT_FROM_LE (len);
         if (len != 16) {
                 close (fd);
                 g_print ("file len not defined\n");
@@ -1292,7 +1200,7 @@ rb_recorder_burn (RBRecorder *recorder,
                 g_set_error (error,
                              RB_RECORDER_ERROR,
                              RB_RECORDER_ERROR_INTERNAL,
-                             _("No writable drives found."));
+                             _("No writable drives found"));
                 return RB_RECORDER_RESULT_ERROR;
         }
 
@@ -1301,7 +1209,7 @@ rb_recorder_burn (RBRecorder *recorder,
                 g_set_error (error,
                              RB_RECORDER_ERROR,
                              RB_RECORDER_ERROR_INTERNAL,
-                             _("Could not determine audio track durations."));
+                             _("Could not determine audio track durations"));
                 return RB_RECORDER_RESULT_ERROR;
         }
 
@@ -1324,25 +1232,17 @@ rb_recorder_burn (RBRecorder *recorder,
                 flags |= NAUTILUS_BURN_RECORDER_WRITE_DEBUG;
         if (TRUE)
                 flags |= NAUTILUS_BURN_RECORDER_WRITE_DISC_AT_ONCE;
+        if (TRUE)
+                flags |= NAUTILUS_BURN_RECORDER_WRITE_BURNPROOF;
 
-	GDK_THREADS_LEAVE ();
-#ifdef NAUTILUS_BURN_DRIVE_SIZE_TO_TIME
-        /* If nautilus-cd-burner >= 2.12 */
+        GDK_THREADS_LEAVE ();
         res = nautilus_burn_recorder_write_tracks (cdrecorder,
                                                    recorder->priv->drive,
                                                    recorder->priv->tracks,
                                                    speed,
                                                    flags,
                                                    &local_error);
-#else
-        /* If nautilus-cd-burner == 2.10 */
-        res = nautilus_burn_recorder_write_tracks (cdrecorder,
-                                                   recorder->priv->drive,
-                                                   recorder->priv->tracks,
-                                                   speed,
-                                                   flags);
-#endif
-	GDK_THREADS_ENTER ();
+        GDK_THREADS_ENTER ();
 
         if (res == NAUTILUS_BURN_RECORDER_RESULT_FINISHED) {
                 result = RB_RECORDER_RESULT_FINISHED;
@@ -1363,14 +1263,14 @@ rb_recorder_burn (RBRecorder *recorder,
                 g_set_error (error,
                              RB_RECORDER_ERROR,
                              RB_RECORDER_ERROR_GENERAL,
-                             msg);
+                             "%s", msg);
                 g_free (msg);
                 result = RB_RECORDER_RESULT_ERROR;
         } else {
                 /* cancelled */
                 result = RB_RECORDER_RESULT_CANCEL;
         }
-        
+
         g_object_unref (cdrecorder);
         recorder->priv->recorder = NULL;
 
