@@ -10,6 +10,14 @@
  *  the Free Software Foundation; either version 2 of the License, or
  *  (at your option) any later version.
  *
+ *  The Rhythmbox authors hereby grant permission for non-GPL compatible
+ *  GStreamer plugins to be used and distributed together with GStreamer
+ *  and Rhythmbox. This permission is above and beyond the permissions granted
+ *  by the GPL license by which Rhythmbox is covered. If you modify this code
+ *  you may extend this exception to your version of the code, but you are not
+ *  obligated to do so. If you do not wish to do so, delete this exception
+ *  statement from your version.
+ *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
@@ -33,6 +41,24 @@
 #include "rb-statusbar.h"
 #include "rb-debug.h"
 
+/**
+ * SECTION:rb-statusbar
+ * @short_description: status bar widget
+ *
+ * The status bar is displayed at the bottom of the main window.  It consists of some
+ * status text and a progress bar.
+ *
+ * The status text usually comes from the selected source, and typically shows the number
+ * of songs, the total duration and the total file size.  When a menu is open, however, 
+ * the status text shows the description of the currently selected menu item.
+ *
+ * The progress bar shows progress information from a variety of sources.  The source that
+ * is currently selected in the source list can provide progress information, such as buffering
+ * feedback, track transfer status, or progress for updating a song catalog.  If the source
+ * does not provide status information and the database is busy (loading the database from disk,
+ * processing a query, etc.) the progress bar will be pulsed periodically.
+ */
+
 #define EPSILON		(0.00001)
 
 static GObject* rb_statusbar_construct (GType type,
@@ -63,7 +89,6 @@ struct RBStatusbarPrivate
         RhythmDB *db;
 
         GtkUIManager *ui_manager;
-        GtkTooltips *tooltips;
 
         GtkWidget *progress;
         double progress_fraction;
@@ -97,6 +122,11 @@ rb_statusbar_class_init (RBStatusbarClass *klass)
         object_class->set_property = rb_statusbar_set_property;
         object_class->get_property = rb_statusbar_get_property;
 
+	/**
+	 * RBStatusbar:db:
+	 *
+	 * The #RhythmDB instance
+	 */
         g_object_class_install_property (object_class,
                                          PROP_DB,
                                          g_param_spec_object ("db",
@@ -104,6 +134,11 @@ rb_statusbar_class_init (RBStatusbarClass *klass)
                                                               "RhythmDB object",
                                                               RHYTHMDB_TYPE,
                                                               G_PARAM_READWRITE));
+	/**
+	 * RBStatusbar:source:
+	 *
+	 * The currently selected #RBSource
+	 */
         g_object_class_install_property (object_class,
                                          PROP_SOURCE,
                                          g_param_spec_object ("source",
@@ -111,6 +146,11 @@ rb_statusbar_class_init (RBStatusbarClass *klass)
                                                               "RBSource object",
                                                               RB_TYPE_SOURCE,
                                                               G_PARAM_READWRITE));
+	/**
+	 * RBStatusbar:ui-manager:
+	 *
+	 * The #GtkUIManager instance
+	 */
         g_object_class_install_property (object_class,
                                          PROP_UI_MANAGER,
                                          g_param_spec_object ("ui-manager",
@@ -149,12 +189,10 @@ rb_statusbar_init (RBStatusbar *statusbar)
 						       RB_TYPE_STATUSBAR,
 						       RBStatusbarPrivate);
 
-        statusbar->priv->tooltips = gtk_tooltips_new ();
-        gtk_tooltips_enable (statusbar->priv->tooltips);
-
         gtk_statusbar_set_has_resize_grip (GTK_STATUSBAR (statusbar), TRUE);
 
         statusbar->priv->progress = gtk_progress_bar_new ();
+	gtk_widget_set_size_request (statusbar->priv->progress, -1, 10);
         statusbar->priv->progress_fraction = 1.0;
 
         statusbar->priv->loading_text = g_strdup (_("Loading..."));
@@ -369,9 +407,15 @@ rb_statusbar_get_property (GObject *object,
         }
 }
 
+/**
+ * rb_statusbar_set_source:
+ * @statusbar: the #RBStatusbar
+ * @source: the new selected #RBSource
+ *
+ * Updates the status bar for a newly selected source.
+ */
 void
-rb_statusbar_set_source (RBStatusbar *statusbar,
-                            RBSource *source)
+rb_statusbar_set_source (RBStatusbar *statusbar, RBSource *source)
 {
         g_return_if_fail (RB_IS_STATUSBAR (statusbar));
         g_return_if_fail (RB_IS_SOURCE (source));
@@ -407,8 +451,17 @@ rb_statusbar_sync_status (RBStatusbar *status)
          * Behaviour of status bar:
 	 * - use source's status text
 	 * - use source's progress value and text, unless internal progress value is set,
-	 *     or the library is busy
+	 * - if no source progress value or internal progress value and library is busy,
+	 *    pulse the progress bar
          */
+        
+	/* library busy? */
+        if (rhythmdb_is_busy (status->priv->db)) {
+		/*g_free (status_text);
+		status_text = g_strdup (status->priv->loading_text); */
+		progress = -1.0f;
+		changed = TRUE;
+        }
 
 	/* get source details */
         if (status->priv->selected_source) {
@@ -419,21 +472,15 @@ rb_statusbar_sync_status (RBStatusbar *status)
 
         /* internal progress bar moving? */
         if (status->priv->progress_fraction < (1.0 - EPSILON) || status->priv->progress_changed) {
-		progress = status->priv->progress_fraction;
+		g_free (progress_text);
                 progress_text = g_strdup (status->priv->progress_text);
+		progress = status->priv->progress_fraction;
+
                 status->priv->progress_changed = FALSE;
 		show_progress = TRUE;
 		changed = TRUE;
         }
 
-        /* library busy? */
-        if (rhythmdb_is_busy (status->priv->db)) {
-		/*g_free (status_text);
-		status_text = g_strdup (status->priv->loading_text); */
-		progress = -1.0f;
-		changed = TRUE;
-        }
-
         /* set up the status text */
         if (status_text) {
                 gtk_statusbar_pop (GTK_STATUSBAR (status), 0);
@@ -464,6 +511,15 @@ rb_statusbar_sync_status (RBStatusbar *status)
                 status->priv->status_poll_id = g_timeout_add (250, (GSourceFunc) poll_status, status);
 }
 
+/**
+ * rb_statusbar_new:
+ * @db: the #RhythmDB instance
+ * @ui_manager: the #GtkUIManager
+ *
+ * Creates the status bar widget.
+ *
+ * Return value: the status bar widget
+ */
 RBStatusbar *
 rb_statusbar_new (RhythmDB *db,
                   GtkUIManager *ui_manager)
@@ -478,6 +534,15 @@ rb_statusbar_new (RhythmDB *db,
         return statusbar;
 }
 
+/**
+ * rb_statusbar_set_progress:
+ * @statusbar: the #RBStatusbar
+ * @progress: progress fraction
+ * @text: text to display on the progress bar
+ *
+ * Updates the progress bar widget.  If the progress fraction is less than zero,
+ * the progress bar is hidden.
+ */
 void
 rb_statusbar_set_progress (RBStatusbar *statusbar, double progress, const char *text)
 {
