@@ -9,6 +9,14 @@
  *  the Free Software Foundation; either version 2 of the License, or
  *  (at your option) any later version.
  *
+ *  The Rhythmbox authors hereby grant permission for non-GPL compatible
+ *  GStreamer plugins to be used and distributed together with GStreamer
+ *  and Rhythmbox. This permission is above and beyond the permissions granted
+ *  by the GPL license by which Rhythmbox is covered. If you modify this code
+ *  you may extend this exception to your version of the code, but you are not
+ *  obligated to do so. If you do not wish to do so, delete this exception
+ *  statement from your version.
+ *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
@@ -20,23 +28,32 @@
  *
  */
 
+/**
+ * SECTION:rb-podcast-source
+ * @short_description: source displaying podcast feeds and episodes
+ *
+ * The podcast source displays podcast episodes in its entry view
+ * and podcast feeds in a property view.  It uses a few custom columns
+ * to display podcast-specific information: episode download status
+ * and an indication of feed parsing errors.
+ */
+
 #include "config.h"
 
 #include <string.h>
 #define __USE_XOPEN
 #include <time.h>
 
-#include <libxml/tree.h>
 #include <glib.h>
 #include <glib/gi18n.h>
 #include <gtk/gtk.h>
-#include <glade/glade.h>
 
 #include "rb-podcast-source.h"
 
+#include "rhythmdb.h"
 #include "rhythmdb-query-model.h"
 #include "rb-statusbar.h"
-#include "rb-glade-helpers.h"
+#include "rb-builder-helpers.h"
 #include "rb-stock-icons.h"
 #include "rb-entry-view.h"
 #include "rb-property-view.h"
@@ -53,13 +70,8 @@
 #include "rb-podcast-manager.h"
 #include "rb-static-playlist-source.h"
 #include "rb-cut-and-paste-code.h"
-
-typedef enum
-{
-	RB_PODCAST_QUERY_TYPE_ALL,
-	RB_PODCAST_QUERY_TYPE_ALBUM,
-	RB_PODCAST_QUERY_TYPE_SEARCH,
-} RBPodcastQueryType;
+#include "rb-source-search-basic.h"
+#include "rb-cell-renderer-pixbuf.h"
 
 static void rb_podcast_source_class_init 		(RBPodcastSourceClass *klass);
 
@@ -181,6 +193,11 @@ static void rb_podcast_source_feed_title_cell_data_func (GtkTreeViewColumn *colu
 						     	 GtkTreeModel *tree_model,
 							 GtkTreeIter *iter,
 						     	 RBPodcastSource *source);
+static void rb_podcast_source_feed_error_cell_data_func (GtkTreeViewColumn *column,
+							 GtkCellRenderer *renderer,
+						     	 GtkTreeModel *tree_model,
+							 GtkTreeIter *iter,
+						     	 RBPodcastSource *source);
 
 static void rb_podcast_source_start_download_cb 	(RBPodcastManager *pd,
 							 RhythmDBEntry *entry,
@@ -194,8 +211,9 @@ static void rb_podcast_source_feed_updates_available_cb (RBPodcastManager *pd,
 							 RhythmDBEntry *entry,
 							 RBPodcastSource *source);
 
-static void rb_podcast_source_download_process_error_cb (RBPodcastManager *pd,
+static gboolean rb_podcast_source_download_process_error_cb (RBPodcastManager *pd,
 							 const char *error,
+							 gboolean existing,
 					  		 RBPodcastSource *source);
 
 static void rb_podcast_source_cb_interval_changed_cb 	(GtkComboBox *box, gpointer cb_data);
@@ -207,18 +225,26 @@ static void rb_podcast_source_entry_activated_cb (RBEntryView *view,
 						  RBPodcastSource *source);
 static void rb_podcast_source_cmd_new_podcast	 (GtkAction *action,
 						  RBPodcastSource *source);
+static void rb_podcast_source_entry_changed_cb	(RhythmDB *db,
+						 RhythmDBEntry *entry,
+						 GSList *changes,
+						 RBPodcastSource *source);
+static void rb_podcast_source_pixbuf_clicked_cb	(RBCellRendererPixbuf *renderer,
+						 const char *path,
+						 RBPodcastSource *source);
 
 /* source methods */
 static char *impl_get_browser_key	 		(RBSource *source);
 static RBEntryView *impl_get_entry_view 		(RBSource *source);
 static void impl_search 				(RBSource *source,
-							 const char *text);
+							 RBSourceSearch *search,
+							 const char *current,
+							 const char *next);
 static void impl_delete 				(RBSource *source);
 static void impl_song_properties 			(RBSource *source);
 static RBSourceEOFType impl_handle_eos 			(RBSource *asource);
 static gboolean impl_show_popup 			(RBSource *source);
-static void rb_podcast_source_do_query			(RBPodcastSource *source,
-							 RBPodcastQueryType type);
+static void rb_podcast_source_do_query			(RBPodcastSource *source);
 static GtkWidget *impl_get_config_widget 		(RBSource *source,
 							 RBShellPreferences *prefs);
 static gboolean impl_receive_drag 			(RBSource *source,
@@ -253,28 +279,25 @@ struct RBPodcastSourcePrivate
 {
 	RhythmDB *db;
 
-	guint toolbar_ui_merge_id;
 	guint prefs_notify_id;
 
 	GtkWidget *vbox;
 	GtkWidget *config_widget;
 	GtkWidget *paned;
 
+	RhythmDBPropertyModel *feed_model;
 	RBPropertyView *feeds;
 	RBEntryView *posts;
 	GtkActionGroup *action_group;
 
-	char *search_text;
 	GList *selected_feeds;
-	RhythmDBQueryModel *cached_all_query;
+	RhythmDBQuery *search_query;
 	RhythmDBPropType search_prop;
+	RBSourceSearch *default_search;
 
-	gboolean initialized;
-
-	RhythmDBEntryType entry_type;
 	RBPodcastManager *podcast_mgr;
 
-	gboolean dispose_has_run;
+	GdkPixbuf *error_pixbuf;
 };
 
 #define RB_PODCAST_SOURCE_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), RB_TYPE_PODCAST_SOURCE, RBPodcastSourcePrivate))
@@ -307,9 +330,9 @@ static GtkActionEntry rb_podcast_source_actions [] =
 
 static GtkRadioActionEntry rb_podcast_source_radio_actions [] =
 {
-	{ "PodcastSearchAll", NULL, N_("All"), NULL, N_("Search all fields"), 0 },
-	{ "PodcastSearchFeeds", NULL, N_("Feeds"), NULL, N_("Search podcast feeds"), 1 },
-	{ "PodcastSearchEpisodes", NULL, N_("Episodes"), NULL, N_("Search podcast episodes"), 2 }
+	{ "PodcastSearchAll", NULL, N_("All"), NULL, N_("Search all fields"), RHYTHMDB_PROP_SEARCH_MATCH },
+	{ "PodcastSearchFeeds", NULL, N_("Feeds"), NULL, N_("Search podcast feeds"), RHYTHMDB_PROP_ALBUM_FOLDED },
+	{ "PodcastSearchEpisodes", NULL, N_("Episodes"), NULL, N_("Search podcast episodes"), RHYTHMDB_PROP_TITLE_FOLDED }
 };
 
 static const GtkTargetEntry posts_view_drag_types[] = {
@@ -321,7 +344,6 @@ static const GtkTargetEntry posts_view_drag_types[] = {
 enum
 {
 	PROP_0,
-	PROP_ENTRY_TYPE,
 	PROP_PODCAST_MANAGER
 };
 
@@ -348,7 +370,6 @@ rb_podcast_source_class_init (RBPodcastSourceClass *klass)
 	source_class->impl_can_cut = (RBSourceFeatureFunc) rb_false_function;
 	source_class->impl_can_delete = (RBSourceFeatureFunc) rb_true_function;
 	source_class->impl_can_pause = (RBSourceFeatureFunc) rb_true_function;
-	source_class->impl_can_search = (RBSourceFeatureFunc) rb_true_function;
 	source_class->impl_delete = impl_delete;
 	source_class->impl_get_browser_key  = impl_get_browser_key;
 	source_class->impl_get_config_widget = impl_get_config_widget;
@@ -378,6 +399,7 @@ rb_podcast_source_class_init (RBPodcastSourceClass *klass)
 static void
 rb_podcast_source_init (RBPodcastSource *source)
 {
+	GtkIconTheme *icon_theme;
 	GdkPixbuf *pixbuf;
 	gint       size;
 
@@ -399,6 +421,13 @@ rb_podcast_source_init (RBPodcastSource *source)
 		rb_source_set_pixbuf (RB_SOURCE (source), pixbuf);
 		g_object_unref (pixbuf);
 	}
+
+	icon_theme = gtk_icon_theme_get_default ();
+	source->priv->error_pixbuf = gtk_icon_theme_load_icon (icon_theme,
+							       "dialog-error",
+							       16,
+							       0,
+							       NULL);
 }
 
 static void
@@ -409,21 +438,14 @@ rb_podcast_source_dispose (GObject *object)
 	rb_debug ("dispose podcast_source");
 	source = RB_PODCAST_SOURCE (object);
 
-	if (source->priv->dispose_has_run) {
-		/* If dispose did already run, return. */
-		return;
-	}
-	/* Make sure dispose does not run twice. */
-	source->priv->dispose_has_run = TRUE;
-
 	if (source->priv->db != NULL) {
 		g_object_unref (source->priv->db);
 		source->priv->db = NULL;
 	}
 
-	if (source->priv->cached_all_query) {
-		g_object_unref (source->priv->cached_all_query);
-		source->priv->cached_all_query = NULL;
+	if (source->priv->search_query != NULL) {
+		rhythmdb_query_free (source->priv->search_query);
+		source->priv->search_query = NULL;
 	}
 
 	if (source->priv->action_group != NULL) {
@@ -436,7 +458,15 @@ rb_podcast_source_dispose (GObject *object)
 		source->priv->podcast_mgr = NULL;
 	}
 
-	eel_gconf_notification_remove (source->priv->prefs_notify_id);
+	if (source->priv->error_pixbuf != NULL) {
+		g_object_unref (source->priv->error_pixbuf);
+		source->priv->error_pixbuf = NULL;
+	}
+
+	if (source->priv->prefs_notify_id != 0) {
+		eel_gconf_notification_remove (source->priv->prefs_notify_id);
+		source->priv->prefs_notify_id = 0;
+	}
 
 	G_OBJECT_CLASS (rb_podcast_source_parent_class)->dispose (object);
 }
@@ -463,53 +493,6 @@ rb_podcast_source_finalize (GObject *object)
 	G_OBJECT_CLASS (rb_podcast_source_parent_class)->finalize (object);
 }
 
-static RhythmDBPropType
-search_action_to_prop (GtkAction *action)
-{
-	const char      *name;
-	RhythmDBPropType prop;
-
-	name = gtk_action_get_name (action);
-
-	if (name == NULL) {
-		prop = RHYTHMDB_PROP_SEARCH_MATCH;
-	} else if (strcmp (name, "PodcastSearchAll") == 0) {
-		prop = RHYTHMDB_PROP_SEARCH_MATCH;
-	} else if (strcmp (name, "PodcastSearchFeeds") == 0) {
-		prop = RHYTHMDB_PROP_ALBUM_FOLDED;
-	} else if (strcmp (name, "PodcastSearchEpisodes") == 0) {
-		prop = RHYTHMDB_PROP_TITLE_FOLDED;
-	} else {
-		prop = RHYTHMDB_PROP_SEARCH_MATCH;
-	}
-
-	return prop;
-}
-
-static void
-search_action_changed (GtkRadioAction  *action,
-		       GtkRadioAction  *current,
-		       RBShell         *shell)
-{
-	gboolean active;
-	RBPodcastSource *source;
-
-	g_object_get (shell, "selected-source", &source, NULL);
-
-	active = gtk_toggle_action_get_active (GTK_TOGGLE_ACTION (current));
-
-	if (active) {
-		/* update query */
-		source->priv->search_prop = search_action_to_prop (GTK_ACTION (current));
-		rb_podcast_source_do_query (source, RB_PODCAST_QUERY_TYPE_SEARCH);
-		rb_source_notify_filter_changed (RB_SOURCE (source));
-	}
-
-	if (source != NULL) {
-		g_object_unref (source);
-	}
-}
-
 static GObject *
 rb_podcast_source_constructor (GType type,
 			       guint n_construct_properties,
@@ -519,10 +502,10 @@ rb_podcast_source_constructor (GType type,
 	RBPodcastSourceClass *klass;
 	GtkTreeViewColumn *column;
 	GtkCellRenderer *renderer;
-	RhythmDBPropertyModel *feed_model;
 	RBShell *shell;
 	RhythmDBQueryModel *query_model;
 	GPtrArray *query;
+	GtkAction *action;
 
 	klass = RB_PODCAST_SOURCE_CLASS (g_type_class_peek (RB_TYPE_PODCAST_SOURCE));
 
@@ -537,12 +520,30 @@ rb_podcast_source_constructor (GType type,
 								       rb_podcast_source_actions,
 								       G_N_ELEMENTS (rb_podcast_source_actions),
 								       source);
+
+	action = gtk_action_group_get_action (source->priv->action_group,
+					      "MusicNewPodcast");
+	/* Translators: this is the toolbar button label
+	   for New Podcast Feed action. */
+	g_object_set (G_OBJECT (action), "short-label", C_("Podcast", "New"), NULL);
+
+	action = gtk_action_group_get_action (source->priv->action_group,
+					      "PodcastUpdateAllFeeds");
+	/* Translators: this is the toolbar button label
+	   for Update All Feeds action. */
+	g_object_set (G_OBJECT (action), "short-label", _("Update"), NULL);
+
 	gtk_action_group_add_radio_actions (source->priv->action_group,
 					    rb_podcast_source_radio_actions,
 					    G_N_ELEMENTS (rb_podcast_source_radio_actions),
 					    0,
-					    (GCallback)search_action_changed,
-					    shell);
+					    NULL,
+					    NULL);
+	rb_source_search_basic_create_for_actions (source->priv->action_group,
+						   rb_podcast_source_radio_actions,
+						   G_N_ELEMENTS (rb_podcast_source_radio_actions));
+
+	source->priv->default_search = rb_source_search_basic_new (RHYTHMDB_PROP_SEARCH_MATCH);
 
 	source->priv->paned = gtk_vpaned_new ();
 
@@ -558,7 +559,7 @@ rb_podcast_source_constructor (GType type,
 						 CONF_STATE_PODCAST_SORTING_POSTS,
 						 TRUE, FALSE);
 
-	g_signal_connect_object (G_OBJECT (source->priv->posts),
+	g_signal_connect_object (source->priv->posts,
 				 "entry-activated",
 				 G_CALLBACK (rb_podcast_source_entry_activated_cb),
 				 source, 0);
@@ -646,36 +647,41 @@ rb_podcast_source_constructor (GType type,
 					    _("Status"), "Status",
 					    (GCompareDataFunc) rb_podcast_source_post_status_cell_sort_func, 0, NULL);
 
-	g_signal_connect_object (G_OBJECT (source->priv->posts),
+	g_signal_connect_object (source->priv->posts,
 				 "sort-order-changed",
 				 G_CALLBACK (rb_podcast_source_posts_view_sort_order_changed_cb),
 				 source, 0);
 
-	g_signal_connect (G_OBJECT (source->priv->podcast_mgr),
+	g_signal_connect (source->priv->podcast_mgr,
 			  "status_changed",
 			  G_CALLBACK (rb_podcast_source_download_status_changed_cb),
 			  source);
 
-	g_signal_connect_object (G_OBJECT (source->priv->podcast_mgr),
+	g_signal_connect_object (source->priv->podcast_mgr,
 			  	 "process_error",
 			 	 G_CALLBACK (rb_podcast_source_download_process_error_cb),
 			  	 source, 0);
 
-	g_signal_connect_object (G_OBJECT (source->priv->posts),
+	g_signal_connect_object (source->priv->posts,
 				 "size_allocate",
 				 G_CALLBACK (paned_size_allocate_cb),
 				 source, 0);
 
-	g_signal_connect_object (G_OBJECT (source->priv->posts), "show_popup",
-				 G_CALLBACK (rb_podcast_source_songs_show_popup_cb), source, 0);
+	g_signal_connect_object (source->priv->posts,
+				 "show_popup",
+				 G_CALLBACK (rb_podcast_source_songs_show_popup_cb),
+				 source, 0);
 
 	/* configure feed view */
-	source->priv->feeds = rb_property_view_new (source->priv->db,  RHYTHMDB_PROP_LOCATION, _("Feed"));
-	rb_property_view_set_selection_mode (RB_PROPERTY_VIEW (source->priv->feeds), GTK_SELECTION_MULTIPLE);
+	source->priv->feeds = rb_property_view_new (source->priv->db,
+						    RHYTHMDB_PROP_LOCATION,
+						    _("Feed"));
+	rb_property_view_set_selection_mode (RB_PROPERTY_VIEW (source->priv->feeds),
+					     GTK_SELECTION_MULTIPLE);
 
 	query_model = rhythmdb_query_model_new_empty (source->priv->db);
-	feed_model = rb_property_view_get_model (RB_PROPERTY_VIEW (source->priv->feeds));
-	g_object_set (feed_model, "query-model", query_model, NULL);
+	source->priv->feed_model = rb_property_view_get_model (RB_PROPERTY_VIEW (source->priv->feeds));
+	g_object_set (source->priv->feed_model, "query-model", query_model, NULL);
 
 	query = rhythmdb_query_parse (source->priv->db,
 				      RHYTHMDB_QUERY_PROP_EQUALS,
@@ -689,7 +695,30 @@ rb_podcast_source_constructor (GType type,
 	rhythmdb_query_free (query);
 	g_object_unref (query_model);
 
-	/* column title */
+	/* error indicator column */
+	column = gtk_tree_view_column_new ();
+	renderer = rb_cell_renderer_pixbuf_new ();
+	gtk_tree_view_column_pack_start (column, renderer, TRUE);
+	gtk_tree_view_column_set_cell_data_func (column, renderer,
+						 (GtkTreeCellDataFunc) rb_podcast_source_feed_error_cell_data_func,
+						 source, NULL);
+	gtk_tree_view_column_set_sizing (column, GTK_TREE_VIEW_COLUMN_FIXED);
+	gtk_tree_view_column_set_fixed_width (column, gdk_pixbuf_get_width (source->priv->error_pixbuf) + 5);
+	gtk_tree_view_column_set_reorderable (column, FALSE);
+	gtk_tree_view_column_set_visible (column, TRUE);
+	rb_property_view_append_column_custom (source->priv->feeds, column);
+	g_signal_connect_object (renderer,
+				 "pixbuf-clicked",
+				 G_CALLBACK (rb_podcast_source_pixbuf_clicked_cb),
+				 source, 0);
+
+	/* redraw error indicator when errors are set or cleared */
+	g_signal_connect_object (source->priv->db,
+				 "entry_changed",
+				 G_CALLBACK (rb_podcast_source_entry_changed_cb),
+				 source, 0);
+
+	/* title column */
 	column = gtk_tree_view_column_new ();
 	renderer = gtk_cell_renderer_text_new ();
 
@@ -740,13 +769,13 @@ rb_podcast_source_constructor (GType type,
 			   posts_view_drag_types, 2,
 			   GDK_ACTION_COPY | GDK_ACTION_MOVE);
 
-	/* set up propiets page */
+	/* set up properties page */
 	gtk_paned_pack1 (GTK_PANED (source->priv->paned),
 			 GTK_WIDGET (source->priv->feeds), FALSE, FALSE);
 	gtk_paned_pack2 (GTK_PANED (source->priv->paned),
 			 GTK_WIDGET (source->priv->posts), TRUE, FALSE);
 
-	gtk_box_pack_start_defaults (GTK_BOX (source->priv->vbox), source->priv->paned);
+	gtk_box_pack_start (GTK_BOX (source->priv->vbox), source->priv->paned, TRUE, TRUE, 0);
 
 	source->priv->prefs_notify_id = eel_gconf_notification_add (CONF_STATE_PODCAST_PREFIX,
 								    (GConfClientNotifyFunc) rb_podcast_source_state_pref_changed,
@@ -755,7 +784,7 @@ rb_podcast_source_constructor (GType type,
 	gtk_widget_show_all (GTK_WIDGET (source));
 	rb_podcast_source_state_prefs_sync (source);
 
-	rb_podcast_source_do_query (source, RB_PODCAST_QUERY_TYPE_ALL);
+	rb_podcast_source_do_query (source);
 
 	return G_OBJECT (source);
 }
@@ -798,6 +827,14 @@ rb_podcast_source_get_property (GObject *object,
 	}
 }
 
+/**
+ * rb_podcast_source_new:
+ * @shell: the #RBShell instance
+ *
+ * Creates the #RBPodcastSource instance
+ *
+ * Return value: the #RBPodcastSource
+ */
 RBSource *
 rb_podcast_source_new (RBShell *shell)
 {
@@ -807,6 +844,7 @@ rb_podcast_source_new (RBShell *shell)
 					  "shell", shell,
 					  "entry-type", RHYTHMDB_ENTRY_TYPE_PODCAST_POST,
 					  "source-group", RB_SOURCE_GROUP_LIBRARY,
+					  "search-type", RB_SOURCE_SEARCH_INCREMENTAL,
 					  NULL));
 
 	rb_shell_register_entry_type_for_source (shell, source,
@@ -818,29 +856,20 @@ rb_podcast_source_new (RBShell *shell)
 }
 
 static void
-impl_search (RBSource *asource, const char *search_text)
+impl_search (RBSource *asource, RBSourceSearch *search, const char *cur_text, const char *new_text)
 {
 	RBPodcastSource *source = RB_PODCAST_SOURCE (asource);
 
-	if (source->priv->initialized) {
-		if (search_text == NULL && source->priv->search_text == NULL)
-			return;
-		if (search_text != NULL &&
-		    source->priv->search_text != NULL
-		    && !strcmp (search_text, source->priv->search_text))
-			return;
+	if (search == NULL) {
+		search = source->priv->default_search;
 	}
 
-	source->priv->initialized = TRUE;
-	if (search_text != NULL && search_text[0] == '\0')
-		search_text = NULL;
-
-	g_free (source->priv->search_text);
-	if (search_text)
-		source->priv->search_text = g_utf8_casefold (search_text, -1);
-	else
-		source->priv->search_text = NULL;
-	rb_podcast_source_do_query (source, RB_PODCAST_QUERY_TYPE_SEARCH);
+	if (source->priv->search_query != NULL) {
+		rhythmdb_query_free (source->priv->search_query);
+		source->priv->search_query = NULL;
+	}
+	source->priv->search_query = rb_source_search_create_query (search, source->priv->db, new_text);
+	rb_podcast_source_do_query (source);
 
 	rb_source_notify_filter_changed (RB_SOURCE (source));
 }
@@ -1082,7 +1111,7 @@ feed_select_change_cb (RBPropertyView *propview,
 
 	source->priv->selected_feeds = rb_string_list_copy (feeds);
 
-	rb_podcast_source_do_query (source, RB_PODCAST_QUERY_TYPE_ALBUM);
+	rb_podcast_source_do_query (source);
 	rb_source_notify_filter_changed (RB_SOURCE (source));
 }
 
@@ -1108,34 +1137,12 @@ construct_query_from_selection (RBPodcastSource *source)
 				      RHYTHMDB_QUERY_END);
 	g_boxed_free (RHYTHMDB_TYPE_ENTRY_TYPE, entry_type);
 
-	if (source->priv->search_text) {
-		GPtrArray *subquery;
-
-		if (source->priv->search_prop == RHYTHMDB_PROP_SEARCH_MATCH) {
-			subquery = rhythmdb_query_parse (source->priv->db,
-							 RHYTHMDB_QUERY_PROP_LIKE,
-							 RHYTHMDB_PROP_ALBUM_FOLDED,
-							 source->priv->search_text,
-							 RHYTHMDB_QUERY_DISJUNCTION,
-							 RHYTHMDB_QUERY_PROP_LIKE,
-							 RHYTHMDB_PROP_TITLE_FOLDED,
-							 source->priv->search_text,
-							 RHYTHMDB_QUERY_END);
-		} else {
-			subquery = rhythmdb_query_parse (source->priv->db,
-							 RHYTHMDB_QUERY_PROP_LIKE,
-							 source->priv->search_prop,
-							 source->priv->search_text,
-							 RHYTHMDB_QUERY_END);
-		}
-
+	if (source->priv->search_query) {
 		rhythmdb_query_append (source->priv->db,
 				       query,
 				       RHYTHMDB_QUERY_SUBQUERY,
-				       subquery,
+				       source->priv->search_query,
 				       RHYTHMDB_QUERY_END);
-
-		rhythmdb_query_free (subquery);
 	}
 
 	if (source->priv->selected_feeds) {
@@ -1170,35 +1177,18 @@ construct_query_from_selection (RBPodcastSource *source)
 }
 
 static void
-rb_podcast_source_do_query (RBPodcastSource *source,
-			    RBPodcastQueryType qtype)
+rb_podcast_source_do_query (RBPodcastSource *source)
 {
 	RhythmDBQueryModel *query_model;
 	GPtrArray *query;
-	gboolean is_all_query;
-
-	rb_debug ("select entry filter");
-
-	is_all_query  = ((qtype == RB_PODCAST_QUERY_TYPE_ALL) ||
-			 ((source->priv->selected_feeds == NULL) &&
-			 (source->priv->search_text == NULL)));
-
-	if (is_all_query && source->priv->cached_all_query) {
-		g_object_unref (source->priv->cached_all_query);
-		source->priv->cached_all_query = NULL;
-	}
 
+	/* set up new query model */
 	query_model = rhythmdb_query_model_new_empty (source->priv->db);
-	if (source->priv->cached_all_query == NULL) {
-		rb_debug ("caching new query");
-		source->priv->cached_all_query = g_object_ref (query_model);
-	}
 
-	rb_debug ("setting empty model");
 	rb_entry_view_set_model (source->priv->posts, query_model);
 	g_object_set (source, "query-model", query_model, NULL);
 
-	rb_debug ("doing query");
+	/* build and run the query */
 	query = construct_query_from_selection (source);
 	rhythmdb_do_full_query_async_parsed (source->priv->db,
 					     RHYTHMDB_QUERY_RESULTS (query_model),
@@ -1220,18 +1210,18 @@ static GtkWidget *
 impl_get_config_widget (RBSource *asource, RBShellPreferences *prefs)
 {
 	RBPodcastSource *source = RB_PODCAST_SOURCE (asource);
+	GtkBuilder *builder;
 	GtkWidget *cb_update_interval;
 	GtkWidget *btn_file;
 	char *download_dir;
-	GladeXML *xml;
 
 	if (source->priv->config_widget)
 		return source->priv->config_widget;
 
-	xml = rb_glade_xml_new ("podcast-prefs.glade", "podcast_vbox", source);
-	source->priv->config_widget = glade_xml_get_widget (xml, "podcast_vbox");
+	builder = rb_builder_load ("podcast-prefs.ui", source);
+	source->priv->config_widget = GTK_WIDGET (gtk_builder_get_object (builder, "podcast_vbox"));
 
-	btn_file = glade_xml_get_widget (xml, "location_chooser");
+	btn_file = GTK_WIDGET (gtk_builder_get_object (builder, "location_chooser"));
 	gtk_file_chooser_add_shortcut_folder (GTK_FILE_CHOOSER (btn_file),
 					      rb_music_dir (),
 					      NULL);
@@ -1245,7 +1235,7 @@ impl_get_config_widget (RBSource *asource, RBShellPreferences *prefs)
 			  G_CALLBACK (rb_podcast_source_btn_file_change_cb),
 			  CONF_STATE_PODCAST_DOWNLOAD_DIR);
 
-	cb_update_interval = glade_xml_get_widget (xml, "cb_update_interval");
+	cb_update_interval = GTK_WIDGET (gtk_builder_get_object (builder, "cb_update_interval"));
 	gtk_combo_box_set_active (GTK_COMBO_BOX (cb_update_interval),
 				  eel_gconf_get_integer (CONF_STATE_PODCAST_DOWNLOAD_INTERVAL));
 	g_signal_connect (cb_update_interval,
@@ -1259,9 +1249,10 @@ impl_get_config_widget (RBSource *asource, RBShellPreferences *prefs)
 static void
 rb_podcast_source_btn_file_change_cb (GtkFileChooserButton *widget, const char *key)
 {
-	char *uri = gtk_file_chooser_get_uri (GTK_FILE_CHOOSER (widget));
-
-	eel_gconf_set_string (key, gnome_vfs_get_local_path_from_uri (uri));
+	char *uri;
+	
+	uri = gtk_file_chooser_get_uri (GTK_FILE_CHOOSER (widget));
+	eel_gconf_set_string (key, uri);
 	g_free (uri);
 }
 
@@ -1278,10 +1269,18 @@ posts_view_drag_data_received_cb (GtkWidget *widget,
 	impl_receive_drag (RB_SOURCE (source), selection_data);
 }
 
+/**
+ * rb_podcast_source_add_feed:
+ * @source: the #RBPodcastSource
+ * @uri: the new feed to add
+ *
+ * Adds a new podcast feed.
+ * Simply calls #rb_podcast_manager_subscribe_feed.
+ */
 void
 rb_podcast_source_add_feed (RBPodcastSource *source, const char *uri)
 {
-	rb_podcast_manager_subscribe_feed (source->priv->podcast_mgr, uri);
+	rb_podcast_manager_subscribe_feed (source->priv->podcast_mgr, uri, FALSE);
 }
 
 static void
@@ -1452,7 +1451,8 @@ rb_podcast_source_cmd_update_feed (GtkAction *action,
 		const char *location = l->data;
 
 		rb_podcast_manager_subscribe_feed (source->priv->podcast_mgr,
-						   location);
+						   location,
+						   FALSE);
 	}
 
 	rb_list_deep_free (feeds);
@@ -1613,6 +1613,32 @@ rb_podcast_source_feed_title_cell_data_func (GtkTreeViewColumn *column,
 	g_free (title);
 }
 
+static void
+rb_podcast_source_feed_error_cell_data_func (GtkTreeViewColumn *column,
+					     GtkCellRenderer *renderer,
+					     GtkTreeModel *tree_model,
+					     GtkTreeIter *iter,
+					     RBPodcastSource *source)
+{
+	char *title;
+	RhythmDBEntry *entry = NULL;
+	GdkPixbuf *pixbuf = NULL;
+
+	gtk_tree_model_get (tree_model, iter,
+			    RHYTHMDB_PROPERTY_MODEL_COLUMN_TITLE, &title,
+			    -1);
+
+	entry = rhythmdb_entry_lookup_by_location (source->priv->db, title);
+	g_free (title);
+
+	if (entry != NULL) {
+		if (rhythmdb_entry_get_string (entry, RHYTHMDB_PROP_PLAYBACK_ERROR)) {
+			pixbuf = source->priv->error_pixbuf;
+		}
+	}
+	g_object_set (renderer, "pixbuf", pixbuf, NULL);
+}
+
 static void
 rb_podcast_source_post_date_cell_data_func (GtkTreeViewColumn *column,
 					    GtkCellRenderer *renderer,
@@ -1677,53 +1703,26 @@ rb_podcast_source_load_finish_cb  (gpointer cb_data)
 static gboolean
 impl_receive_drag (RBSource *asource, GtkSelectionData *selection_data)
 {
-	GList *list, *uri_list, *i;
+	GList *list, *i;
 	RBPodcastSource *source = RB_PODCAST_SOURCE (asource);
 
-	rb_debug ("parsing uri list");
-	list = gnome_vfs_uri_list_parse ((char *)selection_data->data);
+	list = rb_uri_list_parse ((char *)selection_data->data);
 
-	if (list == NULL)
-		return FALSE;
-
-	uri_list = NULL;
-
-	for (i = list; i != NULL; i = g_list_next (i))
-		uri_list = g_list_prepend (uri_list, gnome_vfs_uri_to_string ((const GnomeVFSURI *) i->data, 0));
-
-	gnome_vfs_uri_list_free (list);
-
-	if (uri_list == NULL)
-		return FALSE;
-
-	rb_debug ("adding uris");
-
-	i = uri_list;
-	while (i != NULL) {
+	for (i = list; i != NULL; i = i->next) {
 		char *uri = NULL;
 
-		/* as totem source says, "Super _NETSCAPE_URL trick" */
-		if (selection_data->type == gdk_atom_intern ("_NETSCAPE_URL", FALSE)) {
-			if (i != NULL)
-				g_free (i->data);
-			i = i->next;
-			if (i == NULL)
-				break;
-		}
-
 		uri = i->data;
 		if ((uri != NULL) &&
 		    (!rhythmdb_entry_lookup_by_location (source->priv->db, uri))) {
 			rb_podcast_source_add_feed (source, uri);
 		}
-
-		g_free (uri);
-
-		if (i != NULL)
+		
+		if (selection_data->type == gdk_atom_intern ("_NETSCAPE_URL", FALSE)) {
 			i = i->next;
+		}
 	}
 
-	g_list_free (uri_list);
+	rb_list_deep_free (list);
 	return TRUE;
 }
 
@@ -1733,11 +1732,13 @@ rb_podcast_source_start_download_cb (RBPodcastManager *pd,
 				     RBPodcastSource *source)
 {
 	RBShell *shell = rb_podcast_source_get_shell (source);
-	const gchar *podcast_name = rhythmdb_entry_get_string(entry, RHYTHMDB_PROP_TITLE);
+	char *podcast_name;
+
+	podcast_name = g_markup_escape_text (rhythmdb_entry_get_string (entry, RHYTHMDB_PROP_TITLE), -1);
+	rb_shell_notify_custom (shell, 4000, _("Downloading podcast"), podcast_name, NULL, FALSE);
 
-	rb_debug ("Start download");
-	rb_shell_hidden_notify (shell, 4000, _("Downloading podcast"), NULL, podcast_name, FALSE);
 	g_object_unref (shell);
+	g_free (podcast_name);
 }
 
 static void
@@ -1746,11 +1747,13 @@ rb_podcast_source_finish_download_cb (RBPodcastManager *pd,
 				      RBPodcastSource *source)
 {
 	RBShell *shell = rb_podcast_source_get_shell (source);
-	const gchar *podcast_name = rhythmdb_entry_get_string(entry, RHYTHMDB_PROP_TITLE);
+	char *podcast_name;
+
+	podcast_name = g_markup_escape_text (rhythmdb_entry_get_string (entry, RHYTHMDB_PROP_TITLE), -1);
+	rb_shell_notify_custom (shell, 4000, _("Finished downloading podcast"), podcast_name, NULL, FALSE);
 
-	rb_debug ("Finish download");
-	rb_shell_hidden_notify (shell, 4000, _("Finished downloading podcast"), NULL, podcast_name, FALSE);
 	g_object_unref (shell);
+	g_free (podcast_name);
 }
 
 static void
@@ -1759,11 +1762,13 @@ rb_podcast_source_feed_updates_available_cb (RBPodcastManager *pd,
 					     RBPodcastSource *source)
 {
 	RBShell *shell = rb_podcast_source_get_shell (source);
-	const gchar *podcast_name = rhythmdb_entry_get_string(entry, RHYTHMDB_PROP_TITLE);
+	char *podcast_name;
+
+	podcast_name = g_markup_escape_text (rhythmdb_entry_get_string (entry, RHYTHMDB_PROP_TITLE), -1);
+	rb_shell_notify_custom (shell, 4000, _("New updates available from"), podcast_name, NULL, FALSE);
 
-	rb_debug ("Updates available");
-	rb_shell_hidden_notify (shell, 4000, _("New updates available from"), NULL, podcast_name, FALSE);
 	g_object_unref (shell);
+	g_free (podcast_name);
 
 }
 
@@ -1861,17 +1866,49 @@ rb_podcast_source_post_feed_cell_sort_func (RhythmDBEntry *a,
 	return ret;
 }
 
-static void
+static gboolean
 rb_podcast_source_download_process_error_cb (RBPodcastManager *pd,
 					     const char *error,
+					     gboolean existing,
 					     RBPodcastSource *source)
 {
-	rb_error_dialog (NULL, _("Error in podcast"), "%s", error);
+	GtkWidget *dialog;
+	int result;
+
+	/* if the podcast feed doesn't already exist in the db,
+	 * ask if the user wants to add it anyway; if it already
+	 * exists, there's nothing to do besides reporting the error.
+	 */
+	dialog = gtk_message_dialog_new (GTK_WINDOW (gtk_widget_get_toplevel (GTK_WIDGET (source))),
+					 GTK_DIALOG_DESTROY_WITH_PARENT,
+					 GTK_MESSAGE_ERROR,
+					 existing ? GTK_BUTTONS_OK : GTK_BUTTONS_YES_NO,
+					 _("Error in podcast"));
+
+	if (existing) {
+		gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (dialog),
+							  "%s", error);
+	} else {
+		gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (dialog),
+							  _("%s. Would you like to add the podcast feed anyway?"), error);
+	}
+
+	gtk_window_set_title (GTK_WINDOW (dialog), "");
+	gtk_container_set_border_width (GTK_CONTAINER (dialog), 6);
+
+	result = gtk_dialog_run (GTK_DIALOG (dialog));
+	gtk_widget_destroy (dialog);
+
+	/* in the existing feed case, the response will be _OK or _NONE.
+	 * we want to return FALSE here in this case, so this check works.
+	 */
+	return (result == GTK_RESPONSE_YES);
 }
 
-static void rb_podcast_source_entry_activated_cb (RBEntryView *view,
-						  RhythmDBEntry *entry,
-						  RBPodcastSource *source)
+static void
+rb_podcast_source_entry_activated_cb (RBEntryView *view,
+				      RhythmDBEntry *entry,
+				      RBPodcastSource *source)
 {
 	GValue val = {0,};
 
@@ -1969,7 +2006,7 @@ rb_podcast_source_location_added_cb (RBURIDialog *dialog,
 				     const char *location,
 				     RBPodcastSource *source)
 {
-	rb_podcast_manager_subscribe_feed (source->priv->podcast_mgr, location);
+	rb_podcast_manager_subscribe_feed (source->priv->podcast_mgr, location, FALSE);
 }
 
 static void
@@ -2030,7 +2067,87 @@ static gboolean
 impl_add_uri (RBSource *asource, const char *uri, const char *title, const char *genre)
 {
 	RBPodcastSource *source = RB_PODCAST_SOURCE (asource);
-	rb_podcast_manager_subscribe_feed (source->priv->podcast_mgr, uri);
+	rb_podcast_manager_subscribe_feed (source->priv->podcast_mgr, uri, FALSE);
 	return TRUE;
 }
 
+static void
+rb_podcast_source_entry_changed_cb (RhythmDB *db,
+				    RhythmDBEntry *entry,
+				    GSList *changes,
+				    RBPodcastSource *source)
+{
+	RhythmDBEntryType entry_type;
+	gboolean feed_changed;
+	GSList *t;
+
+	entry_type = rhythmdb_entry_get_entry_type (entry);
+	if (entry_type != RHYTHMDB_ENTRY_TYPE_PODCAST_FEED)
+		return;
+
+	feed_changed = FALSE;
+	for (t = changes; t; t = t->next) {
+		RhythmDBEntryChange *change = t->data;
+
+		if (change->prop == RHYTHMDB_PROP_PLAYBACK_ERROR) {
+			feed_changed = TRUE;
+			break;
+		}
+	}
+
+	if (feed_changed) {
+		const char *loc;
+		GtkTreeIter iter;
+
+		loc = rhythmdb_entry_get_string (entry, RHYTHMDB_PROP_LOCATION);
+		if (rhythmdb_property_model_iter_from_string (source->priv->feed_model,
+							      loc,
+							      &iter)) {
+			GtkTreePath *path;
+
+			path = gtk_tree_model_get_path (GTK_TREE_MODEL (source->priv->feed_model),
+						        &iter);
+			gtk_tree_model_row_changed (GTK_TREE_MODEL (source->priv->feed_model),
+						    path,
+						    &iter);
+			gtk_tree_path_free (path);
+		}
+	}
+}
+
+static void
+rb_podcast_source_pixbuf_clicked_cb (RBCellRendererPixbuf *renderer,
+				     const char *path_string,
+				     RBPodcastSource *source)
+{
+	GtkTreePath *path;
+	GtkTreeIter iter;
+
+	g_return_if_fail (path_string != NULL);
+
+	path = gtk_tree_path_new_from_string (path_string);
+	if (gtk_tree_model_get_iter (GTK_TREE_MODEL (source->priv->feed_model), &iter, path)) {
+		RhythmDBEntry *entry;
+		char *feed_url;
+
+		gtk_tree_model_get (GTK_TREE_MODEL (source->priv->feed_model),
+				    &iter,
+				    RHYTHMDB_PROPERTY_MODEL_COLUMN_TITLE, &feed_url,
+				    -1);
+
+		entry = rhythmdb_entry_lookup_by_location (source->priv->db, feed_url);
+		if (entry != NULL) {
+			const gchar *error;
+
+			error = rhythmdb_entry_get_string (entry, RHYTHMDB_PROP_PLAYBACK_ERROR);
+			if (error) {
+				rb_error_dialog (NULL, _("Podcast Error"), "%s", error);
+			}
+		}
+
+		g_free (feed_url);
+	}
+
+	gtk_tree_path_free (path);
+}
+
