@@ -2,7 +2,7 @@
  *
  *  arch-tag: Implementation of local song properties dialog
  *
- *  Copyright (C) 2002 Olivier Martin <omartin@ifrance.com>
+ *  Copyright (C) 2002 Olivier Martin <olive.martin@gmail.com>
  *  Copyright (C) 2003 Colin Walters <walters@verbum.org>
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -10,7 +10,7 @@
  *  the Free Software Foundation; either version 2 of the License, or
  *  (at your option) any later version.
  *
- *  The Rhythmbox authors hereby grants permission for non-GPL compatible
+ *  The Rhythmbox authors hereby grant permission for non-GPL compatible
  *  GStreamer plugins to be used and distributed together with GStreamer
  *  and Rhythmbox. This permission is above and beyond the permissions granted
  *  by the GPL license by which Rhythmbox is covered. If you modify this code
@@ -44,20 +44,18 @@
 
 #include <glib/gi18n.h>
 #include <gtk/gtk.h>
-#include <glade/glade.h>
-#include <libgnomevfs/gnome-vfs.h>
-#include <libgnomevfs/gnome-vfs-utils.h>
 
 #include "rhythmdb.h"
 #include "rhythmdb-property-model.h"
 #include "rb-song-info.h"
-#include "rb-glade-helpers.h"
+#include "rb-builder-helpers.h"
 #include "rb-dialog.h"
 #include "rb-rating.h"
 #include "rb-preferences.h"
 #include "eel-gconf-extensions.h"
 #include "rb-source.h"
 #include "rb-shell.h"
+#include "rb-file-helpers.h"
 
 static void rb_song_info_class_init (RBSongInfoClass *klass);
 static void rb_song_info_init (RBSongInfo *song_info);
@@ -99,6 +97,9 @@ static void rb_song_info_forward_clicked_cb (GtkWidget *button,
 static void rb_song_info_query_model_changed_cb (GObject *source,
 						 GParamSpec *pspec,
 						 RBSongInfo *song_info);
+static void rb_song_info_base_query_model_changed_cb (GObject *source,
+						      GParamSpec *pspec,
+						      RBSongInfo *song_info);
 static void rb_song_info_rated_cb (RBRating *rating,
 				   double score,
 				   RBSongInfo *song_info);
@@ -111,6 +112,7 @@ struct RBSongInfoPrivate
 	RBSource *source;
 	RBEntryView *entry_view;
 	RhythmDBQueryModel *query_model;
+	RhythmDBQueryModel *base_query_model;
 
 	/* information on the displayed song */
 	RhythmDBEntry *current_entry;
@@ -344,8 +346,7 @@ rb_song_info_show (GtkWidget *widget)
 }
 
 static void
-rb_song_info_construct_single (RBSongInfo *song_info, GladeXML *xml,
-			       gboolean editable)
+rb_song_info_construct_single (RBSongInfo *song_info, GtkBuilder *builder, gboolean editable)
 {
 	song_info->priv->backward = gtk_dialog_add_button (GTK_DIALOG (song_info),
 							   GTK_STOCK_GO_BACK,
@@ -368,28 +369,28 @@ rb_song_info_construct_single (RBSongInfo *song_info, GladeXML *xml,
 	gtk_window_set_title (GTK_WINDOW (song_info), _("Song Properties"));
 
 	/* get the widgets from the XML */
-	song_info->priv->notebook      = glade_xml_get_widget (xml, "song_info_vbox");
-	song_info->priv->title         = glade_xml_get_widget (xml, "song_info_title");
-	song_info->priv->track_cur     = glade_xml_get_widget (xml, "song_info_track_cur");
-	song_info->priv->bitrate       = glade_xml_get_widget (xml, "song_info_bitrate");
-	song_info->priv->duration      = glade_xml_get_widget (xml, "song_info_duration");
-	song_info->priv->location = glade_xml_get_widget (xml, "song_info_location");
-	song_info->priv->filesize = glade_xml_get_widget (xml, "song_info_filesize");
-	song_info->priv->date_added    = glade_xml_get_widget (xml, "song_info_dateadded");
-	song_info->priv->play_count    = glade_xml_get_widget (xml, "song_info_playcount");
-	song_info->priv->last_played   = glade_xml_get_widget (xml, "song_info_lastplayed");
-	song_info->priv->name = glade_xml_get_widget (xml, "song_info_name");
+	song_info->priv->notebook      = GTK_WIDGET (gtk_builder_get_object (builder, "song_info_vbox"));
+	song_info->priv->title         = GTK_WIDGET (gtk_builder_get_object (builder, "song_info_title"));
+	song_info->priv->track_cur     = GTK_WIDGET (gtk_builder_get_object (builder, "song_info_track_cur"));
+	song_info->priv->bitrate       = GTK_WIDGET (gtk_builder_get_object (builder, "song_info_bitrate"));
+	song_info->priv->duration      = GTK_WIDGET (gtk_builder_get_object (builder, "song_info_duration"));
+	song_info->priv->location = GTK_WIDGET (gtk_builder_get_object (builder, "song_info_location"));
+	song_info->priv->filesize = GTK_WIDGET (gtk_builder_get_object (builder, "song_info_filesize"));
+	song_info->priv->date_added    = GTK_WIDGET (gtk_builder_get_object (builder, "song_info_dateadded"));
+	song_info->priv->play_count    = GTK_WIDGET (gtk_builder_get_object (builder, "song_info_playcount"));
+	song_info->priv->last_played   = GTK_WIDGET (gtk_builder_get_object (builder, "song_info_lastplayed"));
+	song_info->priv->name = GTK_WIDGET (gtk_builder_get_object (builder, "song_info_name"));
 
-	rb_glade_boldify_label (xml, "title_label");
-	rb_glade_boldify_label (xml, "trackn_label");
-	rb_glade_boldify_label (xml, "name_label");
-	rb_glade_boldify_label (xml, "location_label");
-	rb_glade_boldify_label (xml, "filesize_label");
-	rb_glade_boldify_label (xml, "date_added_label");
-	rb_glade_boldify_label (xml, "last_played_label");
-	rb_glade_boldify_label (xml, "play_count_label");
-	rb_glade_boldify_label (xml, "duration_label");
-	rb_glade_boldify_label (xml, "bitrate_label");
+	rb_builder_boldify_label (builder, "title_label");
+	rb_builder_boldify_label (builder, "trackn_label");
+	rb_builder_boldify_label (builder, "name_label");
+	rb_builder_boldify_label (builder, "location_label");
+	rb_builder_boldify_label (builder, "filesize_label");
+	rb_builder_boldify_label (builder, "date_added_label");
+	rb_builder_boldify_label (builder, "last_played_label");
+	rb_builder_boldify_label (builder, "play_count_label");
+	rb_builder_boldify_label (builder, "duration_label");
+	rb_builder_boldify_label (builder, "bitrate_label");
 
 	/* whenever you press a mnemonic, the associated GtkEntry's text gets highlighted */
 	g_signal_connect_object (G_OBJECT (song_info->priv->title),
@@ -410,14 +411,13 @@ rb_song_info_construct_single (RBSongInfo *song_info, GladeXML *xml,
 }
 
 static void
-rb_song_info_construct_multiple (RBSongInfo *song_info, GladeXML *xml,
-				 gboolean editable)
+rb_song_info_construct_multiple (RBSongInfo *song_info, GtkBuilder *builder, gboolean editable)
 {
 	gtk_window_set_title (GTK_WINDOW (song_info),
 			      _("Multiple Song Properties"));
 	gtk_widget_grab_focus (song_info->priv->artist);
 
-	song_info->priv->notebook = glade_xml_get_widget (xml, "song_info_notebook");
+	song_info->priv->notebook = GTK_WIDGET (gtk_builder_get_object (builder, "song_info_notebook"));
 }
 
 static void
@@ -438,11 +438,12 @@ rb_song_info_constructor (GType type, guint n_construct_properties,
 {
 	RBSongInfo *song_info;
 	RBSongInfoClass *klass;
-	GladeXML *xml;
 	GList *selected_entries;
 	GList *tem;
 	gboolean editable = TRUE;
 	RBShell *shell;
+	AtkObject *lobj, *robj;
+	GtkBuilder *builder;
 
 	klass = RB_SONG_INFO_CLASS (g_type_class_peek (RB_TYPE_SONG_INFO));
 
@@ -475,39 +476,33 @@ rb_song_info_constructor (GType type, guint n_construct_properties,
 	}
 
 	if (song_info->priv->current_entry) {
-		xml = rb_glade_xml_new ("song-info.glade",
-					"song_info_vbox",
-					song_info);
+		builder = rb_builder_load ("song-info.ui", song_info);
 		gtk_container_add (GTK_CONTAINER (GTK_DIALOG (song_info)->vbox),
-				   glade_xml_get_widget (xml, "song_info_vbox"));
+				   GTK_WIDGET (gtk_builder_get_object (builder, "song_info_vbox")));
 	} else {
-		xml = rb_glade_xml_new ("song-info-multiple.glade",
-					"song_info_notebook",
-					song_info);
+		builder = rb_builder_load ("song-info-multiple.ui", song_info);
 		gtk_container_add (GTK_CONTAINER (GTK_DIALOG (song_info)->vbox),
-				   glade_xml_get_widget (xml, "song_info_notebook"));
+				   GTK_WIDGET (gtk_builder_get_object (builder, "song_info_notebook")));
 	}
 
-	glade_xml_signal_autoconnect (xml);
-
-	song_info->priv->artist = glade_xml_get_widget (xml, "song_info_artist");
-	song_info->priv->album = glade_xml_get_widget (xml, "song_info_album");
-	song_info->priv->genre = glade_xml_get_widget (xml, "song_info_genre");
-	song_info->priv->year = glade_xml_get_widget (xml, "song_info_year");
-	song_info->priv->playback_error_box = glade_xml_get_widget (xml, "song_info_error_box");
-	song_info->priv->playback_error_label = glade_xml_get_widget (xml, "song_info_error_label");
-	song_info->priv->disc_cur = glade_xml_get_widget (xml, "song_info_disc_cur");
+	song_info->priv->artist = GTK_WIDGET (gtk_builder_get_object (builder, "song_info_artist"));
+	song_info->priv->album = GTK_WIDGET (gtk_builder_get_object (builder, "song_info_album"));
+	song_info->priv->genre = GTK_WIDGET (gtk_builder_get_object (builder, "song_info_genre"));
+	song_info->priv->year = GTK_WIDGET (gtk_builder_get_object (builder, "song_info_year"));
+	song_info->priv->playback_error_box = GTK_WIDGET (gtk_builder_get_object (builder, "song_info_error_box"));
+	song_info->priv->playback_error_label = GTK_WIDGET (gtk_builder_get_object (builder, "song_info_error_label"));
+	song_info->priv->disc_cur = GTK_WIDGET (gtk_builder_get_object (builder, "song_info_disc_cur"));
 
 	rb_song_info_add_completion (GTK_ENTRY (song_info->priv->genre), song_info->priv->genres);
 	rb_song_info_add_completion (GTK_ENTRY (song_info->priv->artist), song_info->priv->artists);
 	rb_song_info_add_completion (GTK_ENTRY (song_info->priv->album), song_info->priv->albums);
 
-	rb_glade_boldify_label (xml, "album_label");
-	rb_glade_boldify_label (xml, "artist_label");
-	rb_glade_boldify_label (xml, "genre_label");
-	rb_glade_boldify_label (xml, "year_label");
-	rb_glade_boldify_label (xml, "rating_label");
-	rb_glade_boldify_label (xml, "discn_label");
+	rb_builder_boldify_label (builder, "album_label");
+	rb_builder_boldify_label (builder, "artist_label");
+	rb_builder_boldify_label (builder, "genre_label");
+	rb_builder_boldify_label (builder, "year_label");
+	rb_builder_boldify_label (builder, "rating_label");
+	rb_builder_boldify_label (builder, "discn_label");
 
 	g_signal_connect_object (G_OBJECT (song_info->priv->artist),
 				 "mnemonic-activate",
@@ -535,8 +530,16 @@ rb_song_info_constructor (GType type, guint n_construct_properties,
 	g_signal_connect_object (song_info->priv->rating, "rated",
 				 G_CALLBACK (rb_song_info_rated_cb),
 				 G_OBJECT (song_info), 0);
-	gtk_container_add (GTK_CONTAINER (glade_xml_get_widget (xml, "song_info_rating_container")),
+	gtk_container_add (GTK_CONTAINER (gtk_builder_get_object (builder, "song_info_rating_container")),
 			   song_info->priv->rating);
+	g_object_set (gtk_builder_get_object (builder, "rating_label"), "mnemonic-widget", song_info->priv->rating, NULL);
+
+	/* add relationship between the rating label and the rating widget */
+	lobj = gtk_widget_get_accessible (GTK_WIDGET (gtk_builder_get_object (builder, "rating_label")));
+	robj = gtk_widget_get_accessible (song_info->priv->rating);
+
+	atk_object_add_relationship (lobj, ATK_RELATION_LABEL_FOR, robj);
+	atk_object_add_relationship (robj, ATK_RELATION_LABELLED_BY, lobj);
 
 	gtk_editable_set_editable (GTK_EDITABLE (song_info->priv->artist), editable);
 	gtk_editable_set_editable (GTK_EDITABLE (song_info->priv->album), editable);
@@ -546,10 +549,10 @@ rb_song_info_constructor (GType type, guint n_construct_properties,
 
 	/* Finish construction */
 	if (song_info->priv->current_entry) {
-		rb_song_info_construct_single (song_info, xml, editable);
+		rb_song_info_construct_single (song_info, builder, editable);
 		rb_song_info_populate_dialog (song_info);
 	} else {
-		rb_song_info_construct_multiple (song_info, xml, editable);
+		rb_song_info_construct_multiple (song_info, builder, editable);
 		rb_song_info_populate_dialog_multiple (song_info);
 	}
 	g_object_get (G_OBJECT (song_info->priv->source), "shell", &shell, NULL);
@@ -563,7 +566,8 @@ rb_song_info_constructor (GType type, guint n_construct_properties,
 	gtk_dialog_set_default_response (GTK_DIALOG (song_info),
 					 GTK_RESPONSE_CLOSE);
 
-	g_object_unref (G_OBJECT (xml));
+	g_object_unref (builder);
+
 	return G_OBJECT (song_info);
 }
 
@@ -600,6 +604,9 @@ rb_song_info_dispose (GObject *object)
 		g_signal_handlers_disconnect_by_func (song_info->priv->source,
 						      G_CALLBACK (rb_song_info_query_model_changed_cb),
 						      song_info);
+		g_signal_handlers_disconnect_by_func (song_info->priv->source,
+						      G_CALLBACK (rb_song_info_base_query_model_changed_cb),
+						      song_info);
 		g_object_unref (song_info->priv->source);
 		song_info->priv->source = NULL;
 	}
@@ -635,12 +642,13 @@ static void
 rb_song_info_set_source_internal (RBSongInfo *song_info,
 				  RBSource   *source)
 {
-	RhythmDB *old_db = song_info->priv->db;
-
 	if (song_info->priv->source != NULL) {
 		g_signal_handlers_disconnect_by_func (song_info->priv->source,
 						      rb_song_info_query_model_changed_cb,
 						      song_info);
+		g_signal_handlers_disconnect_by_func (song_info->priv->source,
+						      rb_song_info_base_query_model_changed_cb,
+						      song_info);
 		g_object_unref (song_info->priv->source);
 		g_object_unref (song_info->priv->query_model);
 		g_object_unref (song_info->priv->db);
@@ -656,43 +664,14 @@ rb_song_info_set_source_internal (RBSongInfo *song_info,
 				 "notify::query-model",
 				 G_CALLBACK (rb_song_info_query_model_changed_cb),
 				 song_info, 0);
+	g_signal_connect_object (G_OBJECT (song_info->priv->source),
+				 "notify::base-query-model",
+				 G_CALLBACK (rb_song_info_base_query_model_changed_cb),
+				 song_info, 0);
 
 	g_object_get (G_OBJECT (song_info->priv->query_model), "db", &song_info->priv->db, NULL);
 
-	if (old_db != song_info->priv->db) {
-		if (song_info->priv->albums) {
-			g_object_unref (song_info->priv->albums);
-		}
-		if (song_info->priv->artists) {
-			g_object_unref (song_info->priv->artists);
-		}
-		if (song_info->priv->genres) {
-			g_object_unref (song_info->priv->genres);
-		}
-
-		song_info->priv->albums  = rhythmdb_property_model_new (song_info->priv->db, RHYTHMDB_PROP_ALBUM);
-		song_info->priv->artists = rhythmdb_property_model_new (song_info->priv->db, RHYTHMDB_PROP_ARTIST);
-		song_info->priv->genres  = rhythmdb_property_model_new (song_info->priv->db, RHYTHMDB_PROP_GENRE);
-
-		g_object_set (song_info->priv->albums,  "query-model", song_info->priv->query_model, NULL);
-		g_object_set (song_info->priv->artists, "query-model", song_info->priv->query_model, NULL);
-		g_object_set (song_info->priv->genres,  "query-model", song_info->priv->query_model, NULL);
-
-		if(song_info->priv->album) {
-			GtkEntryCompletion *comp = gtk_entry_get_completion (GTK_ENTRY (song_info->priv->album));
-			gtk_entry_completion_set_model (comp, GTK_TREE_MODEL (song_info->priv->albums));
-		}
-
-		if(song_info->priv->artist) {
-			GtkEntryCompletion *comp = gtk_entry_get_completion (GTK_ENTRY (song_info->priv->artist));
-			gtk_entry_completion_set_model (comp, GTK_TREE_MODEL (song_info->priv->artist));
-		}
-
-		if(song_info->priv->genre) {
-			GtkEntryCompletion *comp = gtk_entry_get_completion (GTK_ENTRY (song_info->priv->genre));
-			gtk_entry_completion_set_model (comp, GTK_TREE_MODEL (song_info->priv->genre));
-		}
-	}
+	rb_song_info_base_query_model_changed_cb (G_OBJECT (song_info->priv->source), NULL, song_info);
 }
 
 static void
@@ -1058,14 +1037,19 @@ rb_song_info_update_playback_error (RBSongInfo *song_info)
 static void
 rb_song_info_update_bitrate (RBSongInfo *song_info)
 {
-	char *tmp = NULL;
-	gulong bitrate = 0;
+	char *tmp;
+	gulong bitrate;
+
 	bitrate = rhythmdb_entry_get_ulong (song_info->priv->current_entry, RHYTHMDB_PROP_BITRATE);
 
-	if (bitrate > 0)
-		tmp = g_strdup_printf (_("%lu kbps"), bitrate);
-	else
+	if (rhythmdb_entry_is_lossless (song_info->priv->current_entry)) {
+		tmp = g_strdup (_("Lossless"));
+	} else if (bitrate == 0) {
 		tmp = g_strdup (_("Unknown"));
+	} else {
+		tmp = g_strdup_printf (_("%lu kbps"), bitrate);
+	}
+
 	gtk_label_set_text (GTK_LABEL (song_info->priv->bitrate),
 			    tmp);
 	g_free (tmp);
@@ -1091,7 +1075,7 @@ rb_song_info_update_filesize (RBSongInfo *song_info)
 	char *text = NULL;
 	guint64 filesize = 0;
 	filesize = rhythmdb_entry_get_uint64 (song_info->priv->current_entry, RHYTHMDB_PROP_FILE_SIZE);
-	text = gnome_vfs_format_file_size_for_display (filesize);
+	text = g_format_size_for_display (filesize);
 	gtk_label_set_text (GTK_LABEL (song_info->priv->filesize), text);
 	g_free (text);
 }
@@ -1110,10 +1094,12 @@ rb_song_info_update_location (RBSongInfo *song_info)
 		char *basename, *dir, *desktopdir;
 
 		basename = g_path_get_basename (text);
-		tmp = gnome_vfs_unescape_string_for_display (basename);
+		tmp = g_uri_unescape_string (basename, NULL);
 		g_free (basename);
 		tmp_utf8 = g_filename_to_utf8 (tmp, -1, NULL, NULL, NULL);
 		g_free (tmp);
+		tmp = NULL;
+
 		if (tmp_utf8 != NULL) {
 			gtk_entry_set_text (GTK_ENTRY (song_info->priv->name),
 					    tmp_utf8);
@@ -1124,19 +1110,22 @@ rb_song_info_update_location (RBSongInfo *song_info)
 
 		g_free (tmp_utf8);
 
-		tmp = gnome_vfs_get_local_path_from_uri (text);
-		if (tmp == NULL)
+		if (rb_uri_is_local (text)) {
+			tmp = g_filename_from_uri (text, NULL, NULL);
+		}
+		if (tmp == NULL) {
 			tmp = g_strdup (text);
+		}
+
 		dir = g_path_get_dirname (tmp);
 		g_free (tmp);
-		tmp = gnome_vfs_unescape_string_for_display (dir);
+		tmp = g_uri_unescape_string (dir, NULL);
 		g_free (dir);
 		tmp_utf8 = g_filename_to_utf8 (tmp, -1, NULL, NULL, NULL);
 		g_free (tmp);
 
 		desktopdir = g_build_filename (g_get_home_dir (), "Desktop", NULL);
-		if ((tmp_utf8 != NULL) && (strcmp (tmp_utf8, desktopdir) == 0))
-		{
+		if ((tmp_utf8 != NULL) && (strcmp (tmp_utf8, desktopdir) == 0)) {
 			g_free (tmp_utf8);
 			tmp_utf8 = g_strdup (_("On the desktop"));
 		}
@@ -1252,6 +1241,51 @@ rb_song_info_query_model_reordered_cb (RhythmDBQueryModel *model,
 	rb_song_info_update_buttons (song_info);
 }
 
+static void
+rb_song_info_base_query_model_changed_cb (GObject *source,
+					  GParamSpec *whatever,
+					  RBSongInfo *song_info)
+{
+	RhythmDBQueryModel *base_query_model;
+
+	g_object_get (source, "base-query-model", &base_query_model, NULL);
+
+	if (song_info->priv->albums) {
+		g_object_unref (song_info->priv->albums);
+	}
+	if (song_info->priv->artists) {
+		g_object_unref (song_info->priv->artists);
+	}
+	if (song_info->priv->genres) {
+		g_object_unref (song_info->priv->genres);
+	}
+
+	song_info->priv->albums  = rhythmdb_property_model_new (song_info->priv->db, RHYTHMDB_PROP_ALBUM);
+	song_info->priv->artists = rhythmdb_property_model_new (song_info->priv->db, RHYTHMDB_PROP_ARTIST);
+	song_info->priv->genres  = rhythmdb_property_model_new (song_info->priv->db, RHYTHMDB_PROP_GENRE);
+
+	g_object_set (song_info->priv->albums,  "query-model", base_query_model, NULL);
+	g_object_set (song_info->priv->artists, "query-model", base_query_model, NULL);
+	g_object_set (song_info->priv->genres,  "query-model", base_query_model, NULL);
+
+	if (song_info->priv->album) {
+		GtkEntryCompletion *comp = gtk_entry_get_completion (GTK_ENTRY (song_info->priv->album));
+		gtk_entry_completion_set_model (comp, GTK_TREE_MODEL (song_info->priv->albums));
+	}
+
+	if (song_info->priv->artist) {
+		GtkEntryCompletion *comp = gtk_entry_get_completion (GTK_ENTRY (song_info->priv->artist));
+		gtk_entry_completion_set_model (comp, GTK_TREE_MODEL (song_info->priv->artist));
+	}
+
+	if (song_info->priv->genre) {
+		GtkEntryCompletion *comp = gtk_entry_get_completion (GTK_ENTRY (song_info->priv->genre));
+		gtk_entry_completion_set_model (comp, GTK_TREE_MODEL (song_info->priv->genre));
+	}
+
+	g_object_unref (base_query_model);
+}
+
 static void
 rb_song_info_query_model_changed_cb (GObject *source,
 				     GParamSpec *whatever,
